/*     */ package com.google.code.yanf4j.core.impl;
/*     */ 
/*     */ import com.google.code.yanf4j.config.Configuration;
/*     */ import com.google.code.yanf4j.core.CodecFactory;
/*     */ import com.google.code.yanf4j.core.Controller;
/*     */ import com.google.code.yanf4j.core.ControllerLifeCycle;
/*     */ import com.google.code.yanf4j.core.ControllerStateListener;
/*     */ import com.google.code.yanf4j.core.Dispatcher;
/*     */ import com.google.code.yanf4j.core.Handler;
/*     */ import com.google.code.yanf4j.core.Session;
/*     */ import com.google.code.yanf4j.core.SocketOption;
/*     */ import com.google.code.yanf4j.core.WriteMessage;
/*     */ import com.google.code.yanf4j.statistics.Statistics;
/*     */ import com.google.code.yanf4j.statistics.impl.DefaultStatistics;
/*     */ import com.google.code.yanf4j.statistics.impl.SimpleStatistics;
/*     */ import com.google.code.yanf4j.util.DispatcherFactory;
/*     */ import com.google.code.yanf4j.util.LinkedTransferQueue;
/*     */ import java.io.IOException;
/*     */ import java.net.InetSocketAddress;
/*     */ import java.nio.channels.SelectionKey;
/*     */ import java.util.Collections;
/*     */ import java.util.HashMap;
/*     */ import java.util.HashSet;
/*     */ import java.util.Map;
/*     */ import java.util.Queue;
/*     */ import java.util.Set;
/*     */ import java.util.concurrent.CopyOnWriteArrayList;
/*     */ import java.util.concurrent.ThreadPoolExecutor.CallerRunsPolicy;
/*     */ import org.slf4j.Logger;
/*     */ import org.slf4j.LoggerFactory;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class AbstractController
/*     */   implements Controller, ControllerLifeCycle
/*     */ {
/*  65 */   protected Statistics statistics = new DefaultStatistics();
/*     */   
/*     */   protected long statisticsInterval;
/*  68 */   protected static final Logger log = LoggerFactory.getLogger(AbstractController.class);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*  73 */   protected CopyOnWriteArrayList<ControllerStateListener> stateListeners = new CopyOnWriteArrayList();
/*     */   
/*     */ 
/*     */   protected Handler handler;
/*     */   
/*     */ 
/*     */   protected volatile CodecFactory codecFactory;
/*     */   
/*     */   protected volatile boolean started;
/*     */   
/*     */   protected InetSocketAddress localSocketAddress;
/*     */   
/*     */   protected int readThreadCount;
/*     */   
/*     */   protected int writeThreadCount;
/*     */   
/*     */   protected int dispatchMessageThreadCount;
/*     */   
/*     */   protected Configuration configuration;
/*     */   
/*     */   protected Dispatcher readEventDispatcher;
/*     */   
/*     */   protected Dispatcher dispatchMessageDispatcher;
/*     */   
/*     */   protected Dispatcher writeEventDispatcher;
/*     */   
/*     */   protected long sessionTimeout;
/*     */   
/* 101 */   protected volatile boolean handleReadWriteConcurrently = true;
/*     */   
/*     */ 
/*     */ 
/*     */   protected int soTimeout;
/*     */   
/*     */ 
/* 108 */   protected Map<SocketOption, Object> socketOptions = new HashMap();
/*     */   
/*     */ 
/*     */   public void setSocketOptions(Map<SocketOption, Object> socketOptions)
/*     */   {
/* 113 */     if (socketOptions == null) {
/* 114 */       throw new NullPointerException("Null socketOptions");
/*     */     }
/* 116 */     this.socketOptions = socketOptions;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/* 122 */   protected Set<Session> sessionSet = new HashSet();
/*     */   private Thread shutdownHookThread;
/* 124 */   private volatile boolean isHutdownHookCalled = false;
/*     */   
/*     */   public final int getDispatchMessageThreadCount() {
/* 127 */     return this.dispatchMessageThreadCount;
/*     */   }
/*     */   
/*     */   public final void setDispatchMessageThreadCount(int dispatchMessageThreadPoolSize)
/*     */   {
/* 132 */     if (this.started) {
/* 133 */       throw new IllegalStateException("Controller is started");
/*     */     }
/* 135 */     if (dispatchMessageThreadPoolSize < 0) {
/* 136 */       throw new IllegalArgumentException("dispatchMessageThreadPoolSize<0");
/*     */     }
/*     */     
/* 139 */     this.dispatchMessageThreadCount = dispatchMessageThreadPoolSize;
/*     */   }
/*     */   
/*     */   public long getSessionIdleTimeout() {
/* 143 */     return this.configuration.getSessionIdleTimeout();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected Queue<WriteMessage> buildQueue()
/*     */   {
/* 152 */     return new LinkedTransferQueue();
/*     */   }
/*     */   
/*     */   public void setSessionIdleTimeout(long sessionIdleTimeout) {
/* 156 */     this.configuration.setSessionIdleTimeout(sessionIdleTimeout);
/*     */   }
/*     */   
/*     */   public long getSessionTimeout()
/*     */   {
/* 161 */     return this.sessionTimeout;
/*     */   }
/*     */   
/*     */   public void setSessionTimeout(long sessionTimeout) {
/* 165 */     this.sessionTimeout = sessionTimeout;
/*     */   }
/*     */   
/*     */   public int getSoTimeout() {
/* 169 */     return this.soTimeout;
/*     */   }
/*     */   
/*     */   public void setSoTimeout(int timeout) {
/* 173 */     this.soTimeout = timeout;
/*     */   }
/*     */   
/*     */   public AbstractController() {
/* 177 */     this(new Configuration(), null, null);
/*     */   }
/*     */   
/*     */   public double getReceiveThroughputLimit() {
/* 181 */     return this.statistics.getReceiveThroughputLimit();
/*     */   }
/*     */   
/*     */   public double getSendThroughputLimit() {
/* 185 */     return this.statistics.getSendThroughputLimit();
/*     */   }
/*     */   
/*     */   public void setReceiveThroughputLimit(double receiveThroughputLimit) {
/* 189 */     this.statistics.setReceiveThroughputLimit(receiveThroughputLimit);
/*     */   }
/*     */   
/*     */   public void setSendThroughputLimit(double sendThroughputLimit)
/*     */   {
/* 194 */     this.statistics.setSendThroughputLimit(sendThroughputLimit);
/*     */   }
/*     */   
/*     */   public AbstractController(Configuration configuration) {
/* 198 */     this(configuration, null, null);
/*     */   }
/*     */   
/*     */ 
/*     */   public AbstractController(Configuration configuration, CodecFactory codecFactory)
/*     */   {
/* 204 */     this(configuration, null, codecFactory);
/*     */   }
/*     */   
/*     */   public AbstractController(Configuration configuration, Handler handler, CodecFactory codecFactory)
/*     */   {
/* 209 */     init(configuration, handler, codecFactory);
/*     */   }
/*     */   
/*     */   private synchronized void init(Configuration configuration, Handler handler, CodecFactory codecFactory)
/*     */   {
/* 214 */     setHandler(handler);
/* 215 */     setCodecFactory(codecFactory);
/* 216 */     setConfiguration(configuration);
/* 217 */     setReadThreadCount(configuration.getReadThreadCount());
/* 218 */     setWriteThreadCount(configuration.getWriteThreadCount());
/* 219 */     setDispatchMessageThreadCount(configuration.getDispatchMessageThreadCount());
/*     */     
/* 221 */     setHandleReadWriteConcurrently(configuration.isHandleReadWriteConcurrently());
/*     */     
/* 223 */     setSoTimeout(configuration.getSoTimeout());
/* 224 */     setStatisticsConfig(configuration);
/* 225 */     setReceiveThroughputLimit(-0.1D);
/* 226 */     setStarted(false);
/*     */   }
/*     */   
/*     */   void setStarted(boolean started) {
/* 230 */     this.started = started;
/*     */   }
/*     */   
/*     */   private void setStatisticsConfig(Configuration configuration) {
/* 234 */     if (configuration.isStatisticsServer()) {
/* 235 */       this.statistics = new SimpleStatistics();
/* 236 */       this.statisticsInterval = configuration.getStatisticsInterval();
/*     */     }
/*     */     else {
/* 239 */       this.statistics = new DefaultStatistics();
/* 240 */       this.statisticsInterval = -1L;
/*     */     }
/*     */   }
/*     */   
/*     */   public Configuration getConfiguration() {
/* 245 */     return this.configuration;
/*     */   }
/*     */   
/*     */   public void setConfiguration(Configuration configuration) {
/* 249 */     if (configuration == null) {
/* 250 */       throw new IllegalArgumentException("Null Configuration");
/*     */     }
/* 252 */     this.configuration = configuration;
/*     */   }
/*     */   
/*     */   public InetSocketAddress getLocalSocketAddress() {
/* 256 */     return this.localSocketAddress;
/*     */   }
/*     */   
/*     */   public void setLocalSocketAddress(InetSocketAddress inetSocketAddress) {
/* 260 */     this.localSocketAddress = inetSocketAddress;
/*     */   }
/*     */   
/*     */   public void onAccept(SelectionKey sk) throws IOException {
/* 264 */     this.statistics.statisticsAccept();
/*     */   }
/*     */   
/*     */   public void onConnect(SelectionKey key) throws IOException {
/* 268 */     throw new UnsupportedOperationException();
/*     */   }
/*     */   
/*     */   public void addStateListener(ControllerStateListener listener) {
/* 272 */     this.stateListeners.add(listener);
/*     */   }
/*     */   
/*     */   public void removeStateListener(ControllerStateListener listener) {
/* 276 */     this.stateListeners.remove(listener);
/*     */   }
/*     */   
/*     */   public boolean isHandleReadWriteConcurrently() {
/* 280 */     return this.handleReadWriteConcurrently;
/*     */   }
/*     */   
/*     */   public void setHandleReadWriteConcurrently(boolean handleReadWriteConcurrently)
/*     */   {
/* 285 */     this.handleReadWriteConcurrently = handleReadWriteConcurrently;
/*     */   }
/*     */   
/*     */   public int getReadThreadCount() {
/* 289 */     return this.readThreadCount;
/*     */   }
/*     */   
/*     */   public void setReadThreadCount(int readThreadCount) {
/* 293 */     if (this.started) {
/* 294 */       throw new IllegalStateException();
/*     */     }
/* 296 */     if (readThreadCount < 0) {
/* 297 */       throw new IllegalArgumentException("readThreadCount<0");
/*     */     }
/* 299 */     this.readThreadCount = readThreadCount;
/*     */   }
/*     */   
/*     */   public final int getWriteThreadCount() {
/* 303 */     return this.writeThreadCount;
/*     */   }
/*     */   
/*     */   public final void setWriteThreadCount(int writeThreadCount) {
/* 307 */     if (this.started) {
/* 308 */       throw new IllegalStateException();
/*     */     }
/* 310 */     if (writeThreadCount < 0) {
/* 311 */       throw new IllegalArgumentException("readThreadCount<0");
/*     */     }
/* 313 */     this.writeThreadCount = writeThreadCount;
/*     */   }
/*     */   
/*     */   public Handler getHandler() {
/* 317 */     return this.handler;
/*     */   }
/*     */   
/*     */   public void setHandler(Handler handler) {
/* 321 */     if (this.started) {
/* 322 */       throw new IllegalStateException("The Controller have started");
/*     */     }
/* 324 */     this.handler = handler;
/*     */   }
/*     */   
/*     */   public int getPort() {
/* 328 */     if (this.localSocketAddress != null) {
/* 329 */       return this.localSocketAddress.getPort();
/*     */     }
/* 331 */     throw new NullPointerException("Controller is not binded");
/*     */   }
/*     */   
/*     */   public synchronized void start() throws IOException {
/* 335 */     if (isStarted()) {
/* 336 */       return;
/*     */     }
/* 338 */     if (getHandler() == null) {
/* 339 */       throw new IOException("The handler is null");
/*     */     }
/* 341 */     if (getCodecFactory() == null) {
/* 342 */       setCodecFactory(new ByteBufferCodecFactory());
/*     */     }
/* 344 */     setStarted(true);
/* 345 */     setReadEventDispatcher(DispatcherFactory.newDispatcher(getReadThreadCount(), new ThreadPoolExecutor.CallerRunsPolicy(), "xmemcached-read-thread"));
/*     */     
/*     */ 
/*     */ 
/* 349 */     setWriteEventDispatcher(DispatcherFactory.newDispatcher(getWriteThreadCount(), new ThreadPoolExecutor.CallerRunsPolicy(), "xmemcached-write-thread"));
/*     */     
/*     */ 
/*     */ 
/* 353 */     setDispatchMessageDispatcher(DispatcherFactory.newDispatcher(getDispatchMessageThreadCount(), new ThreadPoolExecutor.CallerRunsPolicy(), "xmemcached-dispatch-thread"));
/*     */     
/*     */ 
/*     */ 
/* 357 */     startStatistics();
/* 358 */     start0();
/* 359 */     notifyStarted();
/* 360 */     this.shutdownHookThread = new Thread()
/*     */     {
/*     */       public void run() {
/*     */         try {
/* 364 */           AbstractController.this.isHutdownHookCalled = true;
/* 365 */           AbstractController.this.stop();
/*     */         } catch (IOException e) {
/* 367 */           AbstractController.log.error("Stop controller fail", e);
/*     */         }
/*     */       }
/* 370 */     };
/* 371 */     Runtime.getRuntime().addShutdownHook(this.shutdownHookThread);
/* 372 */     log.warn("The Controller started at " + this.localSocketAddress + " ...");
/*     */   }
/*     */   
/*     */   protected abstract void start0() throws IOException;
/*     */   
/*     */   void setDispatchMessageDispatcher(Dispatcher dispatcher) {
/* 378 */     Dispatcher oldDispatcher = this.dispatchMessageDispatcher;
/* 379 */     this.dispatchMessageDispatcher = dispatcher;
/* 380 */     if (oldDispatcher != null) {
/* 381 */       oldDispatcher.stop();
/*     */     }
/*     */   }
/*     */   
/*     */   Dispatcher getReadEventDispatcher() {
/* 386 */     return this.readEventDispatcher;
/*     */   }
/*     */   
/*     */   void setReadEventDispatcher(Dispatcher dispatcher) {
/* 390 */     Dispatcher oldDispatcher = this.readEventDispatcher;
/* 391 */     this.readEventDispatcher = dispatcher;
/* 392 */     if (oldDispatcher != null) {
/* 393 */       oldDispatcher.stop();
/*     */     }
/*     */   }
/*     */   
/*     */   void setWriteEventDispatcher(Dispatcher dispatcher) {
/* 398 */     Dispatcher oldDispatcher = this.writeEventDispatcher;
/* 399 */     this.writeEventDispatcher = dispatcher;
/* 400 */     if (oldDispatcher != null) {
/* 401 */       oldDispatcher.stop();
/*     */     }
/*     */   }
/*     */   
/*     */   private final void startStatistics() {
/* 406 */     this.statistics.start();
/*     */   }
/*     */   
/*     */   public void notifyStarted() {
/* 410 */     for (ControllerStateListener stateListener : this.stateListeners) {
/* 411 */       stateListener.onStarted(this);
/*     */     }
/*     */   }
/*     */   
/*     */   public boolean isStarted() {
/* 416 */     return this.started;
/*     */   }
/*     */   
/*     */   public final Statistics getStatistics() {
/* 420 */     return this.statistics;
/*     */   }
/*     */   
/*     */   public final CodecFactory getCodecFactory() {
/* 424 */     return this.codecFactory;
/*     */   }
/*     */   
/*     */   public final void setCodecFactory(CodecFactory codecFactory) {
/* 428 */     this.codecFactory = codecFactory;
/*     */   }
/*     */   
/*     */   public void notifyReady() {
/* 432 */     for (ControllerStateListener stateListener : this.stateListeners) {
/* 433 */       stateListener.onReady(this);
/*     */     }
/*     */   }
/*     */   
/*     */   public final synchronized void unregisterSession(Session session) {
/* 438 */     this.sessionSet.remove(session);
/* 439 */     if (this.sessionSet.size() == 0) {
/* 440 */       notifyAllSessionClosed();
/* 441 */       notifyAll();
/*     */     }
/*     */   }
/*     */   
/*     */   public void checkStatisticsForRestart() {
/* 446 */     if ((this.statisticsInterval > 0L) && (System.currentTimeMillis() - this.statistics.getStartedTime() > this.statisticsInterval * 1000L))
/*     */     {
/* 448 */       this.statistics.restart();
/*     */     }
/*     */   }
/*     */   
/*     */   public final synchronized void registerSession(Session session) {
/* 453 */     if (this.started) {
/* 454 */       this.sessionSet.add(session);
/*     */     } else {
/* 456 */       session.close();
/*     */     }
/*     */   }
/*     */   
/*     */   public synchronized void stop() throws IOException
/*     */   {
/* 462 */     if (!isStarted()) {
/* 463 */       return;
/*     */     }
/* 465 */     setStarted(false);
/* 466 */     for (Session session : this.sessionSet) {
/* 467 */       session.close();
/*     */     }
/* 469 */     stopStatistics();
/* 470 */     stopDispatcher();
/* 471 */     this.sessionSet.clear();
/* 472 */     notifyStopped();
/* 473 */     clearStateListeners();
/* 474 */     stop0();
/* 475 */     if (!this.isHutdownHookCalled) {
/* 476 */       Runtime.getRuntime().removeShutdownHook(this.shutdownHookThread);
/*     */     }
/* 478 */     log.info("Controller has been stopped.");
/*     */   }
/*     */   
/*     */   protected abstract void stop0()
/*     */     throws IOException;
/*     */   
/*     */   private final void stopDispatcher()
/*     */   {
/* 486 */     if (this.readEventDispatcher != null) {
/* 487 */       this.readEventDispatcher.stop();
/*     */     }
/* 489 */     if (this.dispatchMessageDispatcher != null) {
/* 490 */       this.dispatchMessageDispatcher.stop();
/*     */     }
/* 492 */     if (this.writeEventDispatcher != null) {
/* 493 */       this.writeEventDispatcher.stop();
/*     */     }
/*     */   }
/*     */   
/*     */   private final void stopStatistics() {
/* 498 */     this.statistics.stop();
/*     */   }
/*     */   
/*     */   private final void clearStateListeners() {
/* 502 */     this.stateListeners.clear();
/*     */   }
/*     */   
/*     */   public final void notifyException(Throwable t) {
/* 506 */     for (ControllerStateListener stateListener : this.stateListeners) {
/* 507 */       stateListener.onException(this, t);
/*     */     }
/*     */   }
/*     */   
/*     */   public final void notifyStopped() {
/* 512 */     for (ControllerStateListener stateListener : this.stateListeners) {
/* 513 */       stateListener.onStopped(this);
/*     */     }
/*     */   }
/*     */   
/*     */   public final void notifyAllSessionClosed() {
/* 518 */     for (ControllerStateListener stateListener : this.stateListeners) {
/* 519 */       stateListener.onAllSessionClosed(this);
/*     */     }
/*     */   }
/*     */   
/*     */   public Set<Session> getSessionSet() {
/* 524 */     return Collections.unmodifiableSet(this.sessionSet);
/*     */   }
/*     */   
/*     */   public <T> void setSocketOption(SocketOption<T> socketOption, T value) {
/* 528 */     if (socketOption == null) {
/* 529 */       throw new NullPointerException("Null socketOption");
/*     */     }
/* 531 */     if (value == null) {
/* 532 */       throw new NullPointerException("Null value");
/*     */     }
/* 534 */     if (!socketOption.type().equals(value.getClass())) {
/* 535 */       throw new IllegalArgumentException("Expected " + socketOption.type().getSimpleName() + " value,but givend " + value.getClass().getSimpleName());
/*     */     }
/*     */     
/*     */ 
/* 539 */     this.socketOptions.put(socketOption, value);
/*     */   }
/*     */   
/*     */   public <T> T getSocketOption(SocketOption<T> socketOption)
/*     */   {
/* 544 */     return (T)this.socketOptions.get(socketOption);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void bind(InetSocketAddress inetSocketAddress)
/*     */     throws IOException
/*     */   {
/* 554 */     if (inetSocketAddress == null) {
/* 555 */       throw new IllegalArgumentException("Null inetSocketAddress");
/*     */     }
/* 557 */     setLocalSocketAddress(inetSocketAddress);
/* 558 */     start();
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\google\code\yanf4j\core\impl\AbstractController.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */