/*     */ package com.netapp.asup.data.objects;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.nio.ByteBuffer;
/*     */ import java.util.ArrayList;
/*     */ import java.util.Calendar;
/*     */ import java.util.Collection;
/*     */ import java.util.HashMap;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.Map.Entry;
/*     */ import java.util.TreeMap;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.Schema.Field;
/*     */ import org.apache.avro.Schema.Type;
/*     */ import org.apache.avro.generic.GenericData.Array;
/*     */ import org.apache.avro.generic.GenericData.Record;
/*     */ import org.apache.avro.generic.GenericDatumReader;
/*     */ import org.apache.avro.generic.GenericRecord;
/*     */ import org.apache.avro.util.Utf8;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class LogEventData
/*     */   extends ByteWrappedGenericRecord<Map<Utf8, Map<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>>>, GenericRecord>
/*     */ {
/*     */   private static final String ADD_BIZ_KEY_TO_TAGS = "addBizKeyToTags";
/*     */   private static final String EVENT_NAMES = "eventNames";
/*     */   private final Map<Utf8, Map<Utf8, Map<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>>>> systemToEventMap;
/*     */   private static final String TAGS = "tags";
/*     */   private static final String DOT = ".";
/*     */   private static final String EVENT_MAP = "eventMap";
/*     */   
/*     */   public LogEventData(GenericRecord object, Schema readerSchema, Schema writerSchema, String id)
/*     */     throws IOException
/*     */   {
/*  40 */     super(ARMASchema.LOGEVENTS, writerSchema, readerSchema, id);
/*  41 */     this.object = object;
/*     */     
/*  43 */     this.systemToEventMap = deSerializeMap((Map)object.get("eventMap"), getEmptyEventMap());
/*     */   }
/*     */   
/*     */   public LogEventData(Schema readerSchema, Schema writerSchema, String id) {
/*  47 */     super(ARMASchema.LOGEVENTS, writerSchema, readerSchema, id);
/*  48 */     this.object = new GenericData.Record(ARMASchema.LOGEVENTS.getSchema());
/*  49 */     this.systemToEventMap = getEmptySystemToEventMap();
/*     */   }
/*     */   
/*     */   public Map<Utf8, Map<Utf8, Map<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>>>> getSystemToEventMap() {
/*  53 */     return this.systemToEventMap;
/*     */   }
/*     */   
/*     */   public void addAll(LogEventData logData)
/*     */   {
/*  58 */     this.systemToEventMap.putAll(logData.getSystemToEventMap());
/*     */   }
/*     */   
/*     */   public GenericRecord getObject() throws IOException
/*     */   {
/*  63 */     Map<Utf8, ByteBuffer> serializeMap = serializeMap(this.systemToEventMap);
/*     */     
/*  65 */     Map<Utf8, ByteBuffer> existingSystemToEventMap = (Map)((GenericRecord)this.object).get("eventMap");
/*  66 */     if (existingSystemToEventMap != null) {
/*  67 */       existingSystemToEventMap.putAll(serializeMap);
/*  68 */       ((GenericRecord)this.object).put("eventMap", existingSystemToEventMap);
/*     */     } else {
/*  70 */       ((GenericRecord)this.object).put("eventMap", serializeMap);
/*     */     }
/*     */     
/*  73 */     return (GenericRecord)this.object;
/*     */   }
/*     */   
/*     */   public static Map<Utf8, Map<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>>> getEmptyEventMap() {
/*  77 */     return new HashMap();
/*     */   }
/*     */   
/*     */   public static Map<Utf8, Map<Utf8, Map<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>>>> getEmptySystemToEventMap() {
/*  81 */     return new HashMap();
/*     */   }
/*     */   
/*     */   public void addLogEventData(String requestId, String bizKey, String systemId, byte[] data, Map<Utf8, ObjectRequest> objectDataMap) throws IOException
/*     */   {
/*  86 */     addLogEventData(requestId, bizKey, systemId, this.reader.getExpected(), this.reader.getSchema(), data, objectDataMap);
/*     */   }
/*     */   
/*     */ 
/*     */   public void addLogEventData(String requestId, String bizKey, String systemId, Schema readerSchema, Schema writerSchema, byte[] data, Map<Utf8, ObjectRequest> objectDataMap)
/*     */     throws IOException
/*     */   {
/*  93 */     Map<Utf8, Map<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>>> logData = getEmptyEventMap();
/*  94 */     boolean addBizKeyFlag = false;
/*  95 */     if (null != data) {
/*  96 */       logData = (Map)deserializeData(writerSchema, readerSchema, data);
/*  97 */       for (Map.Entry<Utf8, ObjectRequest> entry : objectDataMap.entrySet()) {
/*  98 */         ObjectRequest objectRequest = (ObjectRequest)entry.getValue();
/*  99 */         GenericRecord logEventRequest = objectRequest.getLogEventData();
/* 100 */         GenericData.Array<Utf8> eventNames = (GenericData.Array)logEventRequest.get("eventNames");
/*     */         
/* 102 */         addBizKeyFlag = ((Boolean)logEventRequest.get("addBizKeyToTags")).booleanValue();
/*     */         
/* 104 */         filterEventTypesAndAddBizKey(logData, (Utf8)entry.getKey(), eventNames, addBizKeyFlag, bizKey);
/*     */       }
/*     */       
/*     */ 
/* 108 */       Utf8 sysId = new Utf8(systemId);
/* 109 */       Map<Utf8, Map<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>>> existingLogData = (Map)deSerializeMap((Map)getObject().get("eventMap"), getEmptyEventMap()).get(sysId);
/*     */       
/* 111 */       if (existingLogData == null) {
/* 112 */         this.systemToEventMap.put(sysId, logData);
/*     */       } else {
/* 114 */         this.systemToEventMap.put(sysId, merge(existingLogData, logData));
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   private Map<Utf8, Map<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>>> merge(Map<Utf8, Map<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>>> existingLogData, Map<Utf8, Map<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>>> logData)
/*     */   {
/* 123 */     for (Map.Entry<Utf8, Map<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>>> objectEntry : logData.entrySet()) {
/* 124 */       Utf8 objectName = (Utf8)objectEntry.getKey();
/* 125 */       Map<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>> instanceMap = (Map)objectEntry.getValue();
/* 126 */       Map<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>> existingInstanceMap = (Map)existingLogData.get(objectName);
/* 127 */       if (existingInstanceMap != null) {
/* 128 */         for (Map.Entry<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>> instanceEntry : instanceMap.entrySet()) {
/* 129 */           Utf8 instanceKey = (Utf8)instanceEntry.getKey();
/* 130 */           Map<Utf8, Map<Utf8, GenericRecord>> eventMap = (Map)instanceEntry.getValue();
/* 131 */           Map<Utf8, Map<Utf8, GenericRecord>> existingEventMap = (Map)existingInstanceMap.get(instanceKey);
/* 132 */           if (existingEventMap != null) {
/* 133 */             for (Map.Entry<Utf8, Map<Utf8, GenericRecord>> eventEntry : eventMap.entrySet()) {
/* 134 */               Utf8 eventType = (Utf8)eventEntry.getKey();
/* 135 */               Map<Utf8, GenericRecord> crcMap = (Map)eventEntry.getValue();
/* 136 */               Map<Utf8, GenericRecord> existingCRCMap = (Map)existingEventMap.get(eventType);
/* 137 */               if (existingCRCMap != null) {
/* 138 */                 for (Map.Entry<Utf8, GenericRecord> crcEntry : crcMap.entrySet()) {
/* 139 */                   Utf8 crcKey = (Utf8)crcEntry.getKey();
/* 140 */                   GenericRecord logRecord = (GenericRecord)crcEntry.getValue();
/* 141 */                   existingCRCMap.put(crcKey, logRecord);
/*     */                 }
/*     */                 
/*     */               }
/*     */               else
/*     */               {
/* 147 */                 existingEventMap.put(eventType, crcMap);
/*     */               }
/*     */               
/*     */             }
/*     */             
/*     */           }
/*     */           else {
/* 154 */             existingInstanceMap.put(instanceKey, eventMap);
/*     */           }
/*     */           
/*     */         }
/*     */         
/*     */       }
/*     */       else {
/* 161 */         existingLogData.put(objectName, instanceMap);
/*     */       }
/*     */     }
/*     */     
/* 165 */     return existingLogData;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private void filterEventTypesAndAddBizKey(Map<Utf8, Map<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>>> logData, Utf8 objectName, GenericData.Array<Utf8> eventNames, boolean addBizKeyFlag, String bizKey)
/*     */   {
/* 181 */     Map<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>> instanceMap = (Map)logData.get(new Utf8(objectName.toString().toLowerCase()));
/*     */     int filterEventSize;
/* 183 */     Map<Utf8, GenericRecord> crcMap; Map<Utf8, Utf8> tagMap; if (instanceMap != null)
/*     */     {
/* 185 */       filterEventSize = eventNames.size();
/* 186 */       crcMap = null;
/* 187 */       tagMap = null;
/* 188 */       for (Utf8 instanceKey : instanceMap.keySet())
/*     */       {
/* 190 */         eventTypeMap = (Map)instanceMap.get(instanceKey);
/*     */         
/* 192 */         List<Utf8> eventTypes = new ArrayList(eventTypeMap.keySet());
/* 193 */         for (Utf8 eventTypeKey : eventTypes)
/*     */         {
/* 195 */           if ((filterEventSize > 0) && (!eventNames.contains(eventTypeKey))) {
/* 196 */             eventTypeMap.remove(eventTypeKey);
/*     */           }
/* 198 */           if (addBizKeyFlag) {
/* 199 */             crcMap = (Map)eventTypeMap.get(eventTypeKey);
/* 200 */             if (crcMap != null) {
/* 201 */               for (GenericRecord record : crcMap.values()) {
/* 202 */                 tagMap = (Map)record.get("tags");
/* 203 */                 if (tagMap == null)
/* 204 */                   tagMap = new HashMap();
/* 205 */                 tagMap.put(new Utf8("biz_key"), new Utf8(bizKey));
/* 206 */                 record.put("tags", tagMap);
/*     */               }
/*     */             }
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     Map<Utf8, Map<Utf8, GenericRecord>> eventTypeMap;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public GenericRecord getRawEventsForCrc(Utf8 systemId, Utf8 objectName, Utf8 instanceName, Utf8 eventType, Utf8 crcKey)
/*     */   {
/* 236 */     GenericRecord eventRecord = null;
/* 237 */     Map<Utf8, Map<Utf8, GenericRecord>> tmpEventTypeMap = getEventsForObjectInstance(systemId, objectName, instanceName);
/*     */     
/* 239 */     if (tmpEventTypeMap != null) {
/* 240 */       Map<Utf8, GenericRecord> tmpCrcMap = (Map)tmpEventTypeMap.get(eventType);
/*     */       
/* 242 */       if (tmpCrcMap != null) {
/* 243 */         eventRecord = (GenericRecord)tmpCrcMap.get(crcKey);
/*     */       }
/*     */     }
/* 246 */     return eventRecord;
/*     */   }
/*     */   
/*     */   public Map<Utf8, Map<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>>> getAllEventsForSystem(Utf8 systemId)
/*     */   {
/* 251 */     return (Map)this.systemToEventMap.get(systemId);
/*     */   }
/*     */   
/*     */   public Map<Utf8, Map<Utf8, GenericRecord>> getEventsForObjectInstance(Utf8 systemId, Utf8 objectName, Utf8 instanceName) {
/* 255 */     Map<Utf8, Map<Utf8, GenericRecord>> eventTypeMap = null;
/* 256 */     Map<Utf8, Map<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>>> logEvent = (Map)this.systemToEventMap.get(systemId);
/*     */     
/* 258 */     if (logEvent != null) {
/* 259 */       Map<Utf8, Map<Utf8, Map<Utf8, GenericRecord>>> tmpInstanceMap = (Map)logEvent.get(objectName);
/*     */       
/* 261 */       if (tmpInstanceMap != null)
/* 262 */         eventTypeMap = (Map)tmpInstanceMap.get(instanceName);
/*     */     }
/* 264 */     return eventTypeMap;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Collection<GenericRecord> getRawEventsForTypes(Utf8 systemId, Utf8 objectName, Utf8 instanceName, String... eventTypes)
/*     */   {
/* 281 */     Collection<GenericRecord> allRecords = new ArrayList();
/* 282 */     Map<Utf8, Map<Utf8, GenericRecord>> tmpEventTypeMap = getEventsForObjectInstance(systemId, objectName, instanceName);
/* 283 */     if ((tmpEventTypeMap != null) && (
/* 284 */       (eventTypes == null) || (eventTypes.length != 0)))
/*     */     {
/*     */ 
/* 287 */       for (String eventType : eventTypes) {
/* 288 */         Utf8 eventTypeUtf8 = new Utf8(eventType);
/* 289 */         Map<Utf8, GenericRecord> tmpCrcMap = (Map)tmpEventTypeMap.get(eventTypeUtf8);
/* 290 */         if (tmpCrcMap != null) {
/* 291 */           allRecords.addAll(tmpCrcMap.values());
/*     */         }
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 298 */     return allRecords;
/*     */   }
/*     */   
/*     */   public static Map<String, String> getLogAttributes(GenericRecord input)
/*     */   {
/* 303 */     Schema s = input.getSchema();
/*     */     
/* 305 */     Map<String, String> logAttributeMap = new TreeMap();
/*     */     
/*     */ 
/*     */ 
/* 309 */     for (Schema.Field f : s.getFields()) {
/* 310 */       String fieldName = f.name();
/* 311 */       Schema.Type fieldType = f.schema().getType();
/* 312 */       switch (fieldType) {
/*     */       case MAP: 
/* 314 */         if (fieldName.equals("tags")) {
/* 315 */           Map<Utf8, Utf8> tagMap = (Map)input.get(fieldName);
/* 316 */           for (Map.Entry<Utf8, Utf8> e : tagMap.entrySet()) {
/* 317 */             logAttributeMap.put("tags." + ((Utf8)e.getKey()).toString(), ((Utf8)e.getValue()).toString());
/*     */           }
/*     */         }
/*     */         else {
/* 321 */           logAttributeMap.put(fieldName, ((String)((Map)input).get(fieldName)).toString());
/*     */         }
/* 323 */         break;
/*     */       case INT: 
/* 325 */         logAttributeMap.put(fieldName, String.valueOf((Integer)input.get(fieldName)));
/* 326 */         break;
/*     */       case LONG: 
/* 328 */         logAttributeMap.put(fieldName, String.valueOf((Long)input.get(fieldName)));
/* 329 */         break;
/*     */       case STRING: 
/* 331 */         logAttributeMap.put(fieldName, input.get(fieldName).toString());
/*     */       default: 
/* 333 */         input.get(fieldName);
/*     */       }
/*     */       
/*     */     }
/* 337 */     return logAttributeMap;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static Collection<GenericRecord> filterEventsByTimestamp(int intervalDays, long currentRequestTimestamp, Collection<GenericRecord> eventRecords)
/*     */   {
/* 354 */     Collection<GenericRecord> filteredRecords = new ArrayList();
/* 355 */     Calendar cal = Calendar.getInstance();
/* 356 */     cal.setTimeInMillis(currentRequestTimestamp);
/* 357 */     cal.add(6, 0 - intervalDays);
/* 358 */     long firstRequestTimestamp = cal.getTimeInMillis();
/* 359 */     for (GenericRecord eventRecord : eventRecords) {
/* 360 */       long firstOccurence = ((Long)eventRecord.get("first_occurence")).longValue();
/* 361 */       if ((firstOccurence > firstRequestTimestamp) && (firstOccurence < currentRequestTimestamp)) {
/* 362 */         filteredRecords.add(eventRecord);
/*     */       }
/*     */     }
/* 365 */     return filteredRecords;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asup\data\objects\LogEventData.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */