/*     */ package com.sun.xml.bind.v2.runtime;
/*     */ 
/*     */ import com.sun.xml.bind.unmarshaller.InfosetScanner;
/*     */ import com.sun.xml.bind.v2.runtime.output.DOMOutput;
/*     */ import com.sun.xml.bind.v2.runtime.unmarshaller.InterningXmlVisitor;
/*     */ import com.sun.xml.bind.v2.runtime.unmarshaller.SAXConnector;
/*     */ import com.sun.xml.bind.v2.runtime.unmarshaller.UnmarshallerImpl;
/*     */ import com.sun.xml.bind.v2.runtime.unmarshaller.UnmarshallingContext;
/*     */ import javax.xml.bind.Binder;
/*     */ import javax.xml.bind.JAXBElement;
/*     */ import javax.xml.bind.JAXBException;
/*     */ import javax.xml.bind.PropertyException;
/*     */ import javax.xml.bind.ValidationEventHandler;
/*     */ import javax.xml.namespace.QName;
/*     */ import javax.xml.validation.Schema;
/*     */ import org.w3c.dom.Element;
/*     */ import org.w3c.dom.Node;
/*     */ import org.xml.sax.SAXException;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class BinderImpl<XmlNode>
/*     */   extends Binder<XmlNode>
/*     */ {
/*     */   private final JAXBContextImpl context;
/*     */   private UnmarshallerImpl unmarshaller;
/*     */   private MarshallerImpl marshaller;
/*     */   private final InfosetScanner<XmlNode> scanner;
/*  75 */   private final AssociationMap<XmlNode> assoc = new AssociationMap();
/*     */   
/*     */   BinderImpl(JAXBContextImpl _context, InfosetScanner<XmlNode> scanner) {
/*  78 */     this.context = _context;
/*  79 */     this.scanner = scanner;
/*     */   }
/*     */   
/*     */   private UnmarshallerImpl getUnmarshaller() {
/*  83 */     if (this.unmarshaller == null)
/*  84 */       this.unmarshaller = new UnmarshallerImpl(this.context, this.assoc);
/*  85 */     return this.unmarshaller;
/*     */   }
/*     */   
/*     */   private MarshallerImpl getMarshaller() {
/*  89 */     if (this.marshaller == null)
/*  90 */       this.marshaller = new MarshallerImpl(this.context, this.assoc);
/*  91 */     return this.marshaller;
/*     */   }
/*     */   
/*     */   public void marshal(Object jaxbObject, XmlNode xmlNode) throws JAXBException {
/*  95 */     if ((xmlNode == null) || (jaxbObject == null))
/*  96 */       throw new IllegalArgumentException();
/*  97 */     getMarshaller().marshal(jaxbObject, createOutput(xmlNode));
/*     */   }
/*     */   
/*     */   private DOMOutput createOutput(XmlNode xmlNode)
/*     */   {
/* 102 */     return new DOMOutput((Node)xmlNode, this.assoc);
/*     */   }
/*     */   
/*     */   public Object updateJAXB(XmlNode xmlNode) throws JAXBException
/*     */   {
/* 107 */     return associativeUnmarshal(xmlNode, true, null);
/*     */   }
/*     */   
/*     */   public Object unmarshal(XmlNode xmlNode) throws JAXBException {
/* 111 */     return associativeUnmarshal(xmlNode, false, null);
/*     */   }
/*     */   
/*     */   public <T> JAXBElement<T> unmarshal(XmlNode xmlNode, Class<T> expectedType) throws JAXBException {
/* 115 */     if (expectedType == null) throw new IllegalArgumentException();
/* 116 */     return (JAXBElement)associativeUnmarshal(xmlNode, true, expectedType);
/*     */   }
/*     */   
/*     */   public void setSchema(Schema schema) {
/* 120 */     getUnmarshaller().setSchema(schema);
/*     */   }
/*     */   
/*     */   public Schema getSchema() {
/* 124 */     return getUnmarshaller().getSchema();
/*     */   }
/*     */   
/*     */   private Object associativeUnmarshal(XmlNode xmlNode, boolean inplace, Class expectedType) throws JAXBException {
/* 128 */     if (xmlNode == null) {
/* 129 */       throw new IllegalArgumentException();
/*     */     }
/* 131 */     JaxBeanInfo bi = null;
/* 132 */     if (expectedType != null) {
/* 133 */       bi = this.context.getBeanInfo(expectedType, true);
/*     */     }
/* 135 */     InterningXmlVisitor handler = new InterningXmlVisitor(getUnmarshaller().createUnmarshallerHandler(this.scanner, inplace, bi));
/*     */     
/* 137 */     this.scanner.setContentHandler(new SAXConnector(handler, this.scanner.getLocator()));
/*     */     try {
/* 139 */       this.scanner.scan(xmlNode);
/*     */     } catch (SAXException e) {
/* 141 */       throw this.unmarshaller.createUnmarshalException(e);
/*     */     }
/*     */     
/* 144 */     return handler.getContext().getResult();
/*     */   }
/*     */   
/*     */   public XmlNode getXMLNode(Object jaxbObject) {
/* 148 */     AssociationMap.Entry<XmlNode> e = this.assoc.byPeer(jaxbObject);
/* 149 */     if (e == null) return null;
/* 150 */     return (XmlNode)e.element();
/*     */   }
/*     */   
/*     */   public Object getJAXBNode(XmlNode xmlNode) {
/* 154 */     AssociationMap.Entry e = this.assoc.byElement(xmlNode);
/* 155 */     if (e == null) return null;
/* 156 */     if (e.outer() != null) return e.outer();
/* 157 */     return e.inner();
/*     */   }
/*     */   
/*     */   public XmlNode updateXML(Object jaxbObject) throws JAXBException {
/* 161 */     return (XmlNode)updateXML(jaxbObject, getXMLNode(jaxbObject));
/*     */   }
/*     */   
/*     */   public XmlNode updateXML(Object jaxbObject, XmlNode xmlNode) throws JAXBException {
/* 165 */     if ((jaxbObject == null) || (xmlNode == null)) { throw new IllegalArgumentException();
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 170 */     Element e = (Element)xmlNode;
/* 171 */     Node ns = e.getNextSibling();
/* 172 */     Node p = e.getParentNode();
/* 173 */     p.removeChild(e);
/*     */     
/*     */ 
/*     */ 
/* 177 */     JaxBeanInfo bi = this.context.getBeanInfo(jaxbObject, true);
/* 178 */     if (!bi.isElement()) {
/* 179 */       jaxbObject = new JAXBElement(new QName(e.getNamespaceURI(), e.getLocalName()), bi.jaxbType, jaxbObject);
/*     */     }
/*     */     
/* 182 */     getMarshaller().marshal(jaxbObject, p);
/* 183 */     Node newNode = p.getLastChild();
/* 184 */     p.removeChild(newNode);
/* 185 */     p.insertBefore(newNode, ns);
/*     */     
/* 187 */     return newNode;
/*     */   }
/*     */   
/*     */   public void setEventHandler(ValidationEventHandler handler) throws JAXBException {
/* 191 */     getUnmarshaller().setEventHandler(handler);
/* 192 */     getMarshaller().setEventHandler(handler);
/*     */   }
/*     */   
/*     */   public ValidationEventHandler getEventHandler() {
/* 196 */     return getUnmarshaller().getEventHandler();
/*     */   }
/*     */   
/*     */   public Object getProperty(String name) throws PropertyException {
/* 200 */     if (name == null) {
/* 201 */       throw new IllegalArgumentException(Messages.NULL_PROPERTY_NAME.format(new Object[0]));
/*     */     }
/*     */     
/* 204 */     if (excludeProperty(name)) {
/* 205 */       throw new PropertyException(name);
/*     */     }
/*     */     
/* 208 */     Object prop = null;
/* 209 */     PropertyException pe = null;
/*     */     try
/*     */     {
/* 212 */       return getMarshaller().getProperty(name);
/*     */     }
/*     */     catch (PropertyException p) {
/* 215 */       pe = p;
/*     */       
/*     */       try
/*     */       {
/* 219 */         return getUnmarshaller().getProperty(name);
/*     */       }
/*     */       catch (PropertyException p) {
/* 222 */         pe = p;
/*     */         
/*     */ 
/* 225 */         pe.setStackTrace(Thread.currentThread().getStackTrace());
/* 226 */         throw pe;
/*     */       }
/*     */     } }
/*     */   
/* 230 */   public void setProperty(String name, Object value) throws PropertyException { if (name == null) {
/* 231 */       throw new IllegalArgumentException(Messages.NULL_PROPERTY_NAME.format(new Object[0]));
/*     */     }
/*     */     
/* 234 */     if (excludeProperty(name)) {
/* 235 */       throw new PropertyException(name, value);
/*     */     }
/*     */     
/* 238 */     PropertyException pe = null;
/*     */     try
/*     */     {
/* 241 */       getMarshaller().setProperty(name, value);
/* 242 */       return;
/*     */     } catch (PropertyException p) {
/* 244 */       pe = p;
/*     */       
/*     */       try
/*     */       {
/* 248 */         getUnmarshaller().setProperty(name, value);
/* 249 */         return;
/*     */       } catch (PropertyException p) {
/* 251 */         pe = p;
/*     */         
/*     */ 
/*     */ 
/*     */ 
/* 256 */         pe.setStackTrace(Thread.currentThread().getStackTrace());
/* 257 */         throw pe;
/*     */       }
/*     */     } }
/*     */   
/* 261 */   private boolean excludeProperty(String name) { return (name.equals("com.sun.xml.bind.characterEscapeHandler")) || (name.equals("com.sun.xml.bind.xmlDeclaration")) || (name.equals("com.sun.xml.bind.xmlHeaders")); }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\target\myautosupport\WEB-INF\lib\jaxb-impl-2.0.3.jar!\com\sun\xml\bind\v2\runtime\BinderImpl.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */