/*     */ package org.apache.avro;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.io.UnsupportedEncodingException;
/*     */ import java.security.MessageDigest;
/*     */ import java.security.NoSuchAlgorithmException;
/*     */ import java.util.HashMap;
/*     */ import java.util.Map;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class SchemaNormalization
/*     */ {
/*     */   static final long EMPTY64 = -4513414715797952619L;
/*     */   
/*     */   public static String toParsingForm(Schema s)
/*     */   {
/*     */     try
/*     */     {
/*  39 */       Map<String, String> env = new HashMap();
/*  40 */       return build(env, s, new StringBuilder()).toString();
/*     */     }
/*     */     catch (IOException e) {
/*  43 */       throw new RuntimeException(e);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static byte[] fingerprint(String fpName, byte[] data)
/*     */     throws NoSuchAlgorithmException
/*     */   {
/*  64 */     if (fpName.equals("CRC-64-AVRO")) {
/*  65 */       long fp = fingerprint64(data);
/*  66 */       byte[] result = new byte[8];
/*  67 */       for (int i = 0; i < 8; i++) {
/*  68 */         result[i] = ((byte)(int)fp);
/*  69 */         fp >>= 8;
/*     */       }
/*  71 */       return result;
/*     */     }
/*     */     
/*  74 */     MessageDigest md = MessageDigest.getInstance(fpName);
/*  75 */     return md.digest(data);
/*     */   }
/*     */   
/*     */ 
/*     */   public static long fingerprint64(byte[] data)
/*     */   {
/*  81 */     long result = -4513414715797952619L;
/*  82 */     for (byte b : data)
/*  83 */       result = result >>> 8 ^ FP64.FP_TABLE[((int)(result ^ b) & 0xFF)];
/*  84 */     return result;
/*     */   }
/*     */   
/*     */ 
/*     */   public static byte[] parsingFingerprint(String fpName, Schema s)
/*     */     throws NoSuchAlgorithmException
/*     */   {
/*     */     try
/*     */     {
/*  93 */       return fingerprint(fpName, toParsingForm(s).getBytes("UTF-8"));
/*  94 */     } catch (UnsupportedEncodingException e) { throw new RuntimeException(e);
/*     */     }
/*     */   }
/*     */   
/*     */   public static long parsingFingerprint64(Schema s)
/*     */   {
/*     */     try {
/* 101 */       return fingerprint64(toParsingForm(s).getBytes("UTF-8"));
/*     */     } catch (UnsupportedEncodingException e) {
/* 103 */       throw new RuntimeException(e);
/*     */     }
/*     */   }
/*     */   
/*     */   private static Appendable build(Map<String, String> env, Schema s, Appendable o) throws IOException {
/* 108 */     boolean firstTime = true;
/* 109 */     Schema.Type st = s.getType();
/* 110 */     switch (st) {
/*     */     default: 
/* 112 */       return o.append('"').append(st.getName()).append('"');
/*     */     
/*     */     case UNION: 
/* 115 */       o.append('[');
/* 116 */       for (Schema b : s.getTypes()) {
/* 117 */         if (!firstTime) o.append(','); else firstTime = false;
/* 118 */         build(env, b, o);
/*     */       }
/* 120 */       return o.append(']');
/*     */     case ARRAY: 
/*     */     case MAP: 
/* 123 */       o.append("{\"type\":\"").append(st.getName()).append("\"");
/* 124 */       if (st == Schema.Type.ARRAY)
/* 125 */         build(env, s.getElementType(), o.append(",\"items\":")); else
/* 126 */         build(env, s.getValueType(), o.append(",\"values\":"));
/* 127 */       return o.append("}");
/*     */     }
/*     */     
/* 130 */     String name = s.getFullName();
/* 131 */     if (env.get(name) != null) return o.append((CharSequence)env.get(name));
/* 132 */     String qname = "\"" + name + "\"";
/* 133 */     env.put(name, qname);
/* 134 */     o.append("{\"name\":").append(qname);
/* 135 */     o.append(",\"type\":\"").append(st.getName()).append("\"");
/* 136 */     if (st == Schema.Type.ENUM) {
/* 137 */       o.append(",\"symbols\":[");
/* 138 */       for (String enumSymbol : s.getEnumSymbols()) {
/* 139 */         if (!firstTime) o.append(','); else firstTime = false;
/* 140 */         o.append('"').append(enumSymbol).append('"');
/*     */       }
/* 142 */       o.append("]");
/* 143 */     } else if (st == Schema.Type.FIXED) {
/* 144 */       o.append(",\"size\":").append(Integer.toString(s.getFixedSize()));
/*     */     } else {
/* 146 */       o.append(",\"fields\":[");
/* 147 */       for (Schema.Field f : s.getFields()) {
/* 148 */         if (!firstTime) o.append(','); else firstTime = false;
/* 149 */         o.append("{\"name\":\"").append(f.name()).append("\"");
/* 150 */         build(env, f.schema(), o.append(",\"type\":")).append("}");
/*     */       }
/* 152 */       o.append("]");
/*     */     }
/* 154 */     return o.append("}");
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   private static class FP64
/*     */   {
/* 162 */     private static final long[] FP_TABLE = new long['Ā'];
/*     */     
/* 164 */     static { for (int i = 0; i < 256; i++) {
/* 165 */         long fp = i;
/* 166 */         for (int j = 0; j < 8; j++) {
/* 167 */           long mask = -(fp & 1L);
/* 168 */           fp = fp >>> 1 ^ 0xC15D213AA4D7A795 & mask;
/*     */         }
/* 170 */         FP_TABLE[i] = fp;
/*     */       }
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\SchemaNormalization.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */