/*     */ package org.apache.avro.io.parsing;
/*     */ 
/*     */ import java.io.ByteArrayOutputStream;
/*     */ import java.io.IOException;
/*     */ import java.util.Arrays;
/*     */ import java.util.HashMap;
/*     */ import java.util.Iterator;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import org.apache.avro.AvroTypeException;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.Schema.Field;
/*     */ import org.apache.avro.Schema.Type;
/*     */ import org.apache.avro.io.Encoder;
/*     */ import org.apache.avro.io.EncoderFactory;
/*     */ import org.codehaus.jackson.JsonNode;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class ResolvingGrammarGenerator
/*     */   extends ValidatingGrammarGenerator
/*     */ {
/*     */   public final Symbol generate(Schema writer, Schema reader)
/*     */     throws IOException
/*     */   {
/*  50 */     return Symbol.root(new Symbol[] { generate(writer, reader, new HashMap()) });
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Symbol generate(Schema writer, Schema reader, Map<ValidatingGrammarGenerator.LitS, Symbol> seen)
/*     */     throws IOException
/*     */   {
/*  69 */     Schema.Type writerType = writer.getType();
/*  70 */     Schema.Type readerType = reader.getType();
/*     */     
/*  72 */     if (writerType == readerType) {
/*  73 */       switch (writerType) {
/*     */       case NULL: 
/*  75 */         return Symbol.NULL;
/*     */       case BOOLEAN: 
/*  77 */         return Symbol.BOOLEAN;
/*     */       case INT: 
/*  79 */         return Symbol.INT;
/*     */       case LONG: 
/*  81 */         return Symbol.LONG;
/*     */       case FLOAT: 
/*  83 */         return Symbol.FLOAT;
/*     */       case DOUBLE: 
/*  85 */         return Symbol.DOUBLE;
/*     */       case STRING: 
/*  87 */         return Symbol.STRING;
/*     */       case BYTES: 
/*  89 */         return Symbol.BYTES;
/*     */       case FIXED: 
/*  91 */         if ((!writer.getFullName().equals(reader.getFullName())) || (writer.getFixedSize() != reader.getFixedSize()))
/*     */           break;
/*  93 */         return Symbol.seq(new Symbol[] { new Symbol.IntCheckAction(writer.getFixedSize()), Symbol.FIXED });
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */       case ENUM: 
/*  99 */         if ((writer.getFullName() != null) && (!writer.getFullName().equals(reader.getFullName())))
/*     */           break;
/* 101 */         return Symbol.seq(new Symbol[] { mkEnumAdjust(writer.getEnumSymbols(), reader.getEnumSymbols()), Symbol.ENUM });
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */       case ARRAY: 
/* 107 */         return Symbol.seq(new Symbol[] { Symbol.repeat(Symbol.ARRAY_END, new Symbol[] { generate(writer.getElementType(), reader.getElementType(), seen) }), Symbol.ARRAY_START });
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */       case MAP: 
/* 113 */         return Symbol.seq(new Symbol[] { Symbol.repeat(Symbol.MAP_END, new Symbol[] { generate(writer.getValueType(), reader.getValueType(), seen), Symbol.STRING }), Symbol.MAP_START });
/*     */       
/*     */ 
/*     */ 
/*     */       case RECORD: 
/* 118 */         return resolveRecords(writer, reader, seen);
/*     */       case UNION: 
/* 120 */         return resolveUnion(writer, reader, seen);
/*     */       default: 
/* 122 */         throw new AvroTypeException("Unkown type for schema: " + writerType);
/*     */       }
/*     */     } else {
/* 125 */       if (writerType == Schema.Type.UNION) {
/* 126 */         return resolveUnion(writer, reader, seen);
/*     */       }
/*     */       
/* 129 */       switch (readerType) {
/*     */       case LONG: 
/* 131 */         switch (writerType) {
/*     */         case INT: 
/* 133 */           return Symbol.resolve(super.generate(writer, seen), Symbol.LONG);
/*     */         }
/* 135 */         break;
/*     */       
/*     */       case FLOAT: 
/* 138 */         switch (writerType) {
/*     */         case INT: 
/*     */         case LONG: 
/* 141 */           return Symbol.resolve(super.generate(writer, seen), Symbol.FLOAT);
/*     */         }
/* 143 */         break;
/*     */       
/*     */       case DOUBLE: 
/* 146 */         switch (writerType) {
/*     */         case INT: 
/*     */         case LONG: 
/*     */         case FLOAT: 
/* 150 */           return Symbol.resolve(super.generate(writer, seen), Symbol.DOUBLE);
/*     */         }
/* 152 */         break;
/*     */       
/*     */       case UNION: 
/* 155 */         int j = bestBranch(reader, writer);
/* 156 */         if (j >= 0) {
/* 157 */           Symbol s = generate(writer, (Schema)reader.getTypes().get(j), seen);
/* 158 */           return Symbol.seq(new Symbol[] { new Symbol.UnionAdjustAction(j, s), Symbol.UNION });
/*     */         }
/*     */         break;
/*     */       case NULL: 
/*     */       case BOOLEAN: 
/*     */       case INT: 
/*     */       case STRING: 
/*     */       case BYTES: 
/*     */       case ENUM: 
/*     */       case ARRAY: 
/*     */       case MAP: 
/*     */       case RECORD: 
/*     */         break;
/*     */       case FIXED: default: 
/* 172 */         throw new RuntimeException("Unexpected schema type: " + readerType);
/*     */       }
/*     */     }
/* 175 */     return Symbol.error("Found " + writer.toString(true) + ", expecting " + reader.toString(true));
/*     */   }
/*     */   
/*     */   private Symbol resolveUnion(Schema writer, Schema reader, Map<ValidatingGrammarGenerator.LitS, Symbol> seen)
/*     */     throws IOException
/*     */   {
/* 181 */     List<Schema> alts = writer.getTypes();
/* 182 */     int size = alts.size();
/* 183 */     Symbol[] symbols = new Symbol[size];
/* 184 */     String[] labels = new String[size];
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 190 */     int i = 0;
/* 191 */     for (Schema w : alts) {
/* 192 */       symbols[i] = generate(w, reader, seen);
/* 193 */       labels[i] = w.getFullName();
/* 194 */       i++;
/*     */     }
/* 196 */     return Symbol.seq(new Symbol[] { Symbol.alt(symbols, labels), new Symbol.WriterUnionAction() });
/*     */   }
/*     */   
/*     */   private Symbol resolveRecords(Schema writer, Schema reader, Map<ValidatingGrammarGenerator.LitS, Symbol> seen)
/*     */     throws IOException
/*     */   {
/* 202 */     ValidatingGrammarGenerator.LitS wsc = new LitS2(writer, reader);
/* 203 */     Symbol result = (Symbol)seen.get(wsc);
/* 204 */     int count; Symbol[] production; if (result == null) {
/* 205 */       List<Schema.Field> wfields = writer.getFields();
/* 206 */       List<Schema.Field> rfields = reader.getFields();
/*     */       
/*     */ 
/*     */ 
/* 210 */       Schema.Field[] reordered = new Schema.Field[rfields.size()];
/* 211 */       int ridx = 0;
/* 212 */       count = 1 + wfields.size();
/*     */       
/* 214 */       for (Schema.Field f : wfields) {
/* 215 */         Schema.Field rdrField = reader.getField(f.name());
/* 216 */         if (rdrField != null) {
/* 217 */           reordered[(ridx++)] = rdrField;
/*     */         }
/*     */       }
/*     */       
/* 221 */       for (Schema.Field rf : rfields) {
/* 222 */         String fname = rf.name();
/* 223 */         if (writer.getField(fname) == null) {
/* 224 */           if (rf.defaultValue() == null) {
/* 225 */             result = Symbol.error("Found " + writer.toString(true) + ", expecting " + reader.toString(true));
/*     */             
/* 227 */             seen.put(wsc, result);
/* 228 */             return result;
/*     */           }
/* 230 */           reordered[(ridx++)] = rf;
/* 231 */           count += 3;
/*     */         }
/*     */       }
/*     */       
/*     */ 
/* 236 */       production = new Symbol[count];
/* 237 */       production[(--count)] = new Symbol.FieldOrderAction(reordered);
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 243 */       result = Symbol.seq(production);
/* 244 */       seen.put(wsc, result);
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 254 */       for (Schema.Field wf : wfields) {
/* 255 */         String fname = wf.name();
/* 256 */         Schema.Field rf = reader.getField(fname);
/* 257 */         if (rf == null) {
/* 258 */           production[(--count)] = new Symbol.SkipAction(generate(wf.schema(), wf.schema(), seen));
/*     */         }
/*     */         else {
/* 261 */           production[(--count)] = generate(wf.schema(), rf.schema(), seen);
/*     */         }
/*     */       }
/*     */       
/*     */ 
/*     */ 
/* 267 */       for (Schema.Field rf : rfields) {
/* 268 */         String fname = rf.name();
/* 269 */         Schema.Field wf = writer.getField(fname);
/* 270 */         if (wf == null) {
/* 271 */           byte[] bb = getBinary(rf.schema(), rf.defaultValue());
/* 272 */           production[(--count)] = new Symbol.DefaultStartAction(bb);
/* 273 */           production[(--count)] = generate(rf.schema(), rf.schema(), seen);
/* 274 */           production[(--count)] = Symbol.DEFAULT_END_ACTION;
/*     */         }
/*     */       }
/*     */     }
/* 278 */     return result;
/*     */   }
/*     */   
/* 281 */   private static EncoderFactory factory = new EncoderFactory().configureBufferSize(32);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   private static byte[] getBinary(Schema s, JsonNode n)
/*     */     throws IOException
/*     */   {
/* 291 */     ByteArrayOutputStream out = new ByteArrayOutputStream();
/* 292 */     Encoder e = factory.binaryEncoder(out, null);
/* 293 */     encode(e, s, n);
/* 294 */     e.flush();
/* 295 */     return out.toByteArray();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static void encode(Encoder e, Schema s, JsonNode n)
/*     */     throws IOException
/*     */   {
/* 309 */     switch (s.getType()) {
/*     */     case RECORD: 
/* 311 */       for (Schema.Field f : s.getFields()) {
/* 312 */         String name = f.name();
/* 313 */         JsonNode v = n.get(name);
/* 314 */         if (v == null) {
/* 315 */           v = f.defaultValue();
/*     */         }
/* 317 */         if (v == null) {
/* 318 */           throw new AvroTypeException("No default value for: " + name);
/*     */         }
/* 320 */         encode(e, f.schema(), v);
/*     */       }
/* 322 */       break;
/*     */     case ENUM: 
/* 324 */       e.writeEnum(s.getEnumOrdinal(n.getTextValue()));
/* 325 */       break;
/*     */     case ARRAY: 
/* 327 */       e.writeArrayStart();
/* 328 */       e.setItemCount(n.size());
/* 329 */       Schema i = s.getElementType();
/* 330 */       for (JsonNode node : n) {
/* 331 */         e.startItem();
/* 332 */         encode(e, i, node);
/*     */       }
/* 334 */       e.writeArrayEnd();
/* 335 */       break;
/*     */     case MAP: 
/* 337 */       e.writeMapStart();
/* 338 */       e.setItemCount(n.size());
/* 339 */       Schema v = s.getValueType();
/* 340 */       for (Iterator<String> it = n.getFieldNames(); it.hasNext();) {
/* 341 */         e.startItem();
/* 342 */         String key = (String)it.next();
/* 343 */         e.writeString(key);
/* 344 */         encode(e, v, n.get(key));
/*     */       }
/* 346 */       e.writeMapEnd();
/* 347 */       break;
/*     */     case UNION: 
/* 349 */       e.writeIndex(0);
/* 350 */       encode(e, (Schema)s.getTypes().get(0), n);
/* 351 */       break;
/*     */     case FIXED: 
/* 353 */       if (!n.isTextual())
/* 354 */         throw new AvroTypeException("Non-string default value for fixed: " + n);
/* 355 */       byte[] bb = n.getTextValue().getBytes("ISO-8859-1");
/* 356 */       if (bb.length != s.getFixedSize()) {
/* 357 */         bb = Arrays.copyOf(bb, s.getFixedSize());
/*     */       }
/* 359 */       e.writeFixed(bb);
/* 360 */       break;
/*     */     case STRING: 
/* 362 */       if (!n.isTextual())
/* 363 */         throw new AvroTypeException("Non-string default value for string: " + n);
/* 364 */       e.writeString(n.getTextValue());
/* 365 */       break;
/*     */     case BYTES: 
/* 367 */       if (!n.isTextual())
/* 368 */         throw new AvroTypeException("Non-string default value for bytes: " + n);
/* 369 */       e.writeBytes(n.getTextValue().getBytes("ISO-8859-1"));
/* 370 */       break;
/*     */     case INT: 
/* 372 */       if (!n.isNumber())
/* 373 */         throw new AvroTypeException("Non-numeric default value for int: " + n);
/* 374 */       e.writeInt(n.getIntValue());
/* 375 */       break;
/*     */     case LONG: 
/* 377 */       if (!n.isNumber())
/* 378 */         throw new AvroTypeException("Non-numeric default value for long: " + n);
/* 379 */       e.writeLong(n.getLongValue());
/* 380 */       break;
/*     */     case FLOAT: 
/* 382 */       if (!n.isNumber())
/* 383 */         throw new AvroTypeException("Non-numeric default value for float: " + n);
/* 384 */       e.writeFloat((float)n.getDoubleValue());
/* 385 */       break;
/*     */     case DOUBLE: 
/* 387 */       if (!n.isNumber())
/* 388 */         throw new AvroTypeException("Non-numeric default value for double: " + n);
/* 389 */       e.writeDouble(n.getDoubleValue());
/* 390 */       break;
/*     */     case BOOLEAN: 
/* 392 */       if (!n.isBoolean())
/* 393 */         throw new AvroTypeException("Non-boolean default for boolean: " + n);
/* 394 */       e.writeBoolean(n.getBooleanValue());
/* 395 */       break;
/*     */     case NULL: 
/* 397 */       if (!n.isNull())
/* 398 */         throw new AvroTypeException("Non-null default value for null type: " + n);
/* 399 */       e.writeNull();
/*     */     }
/*     */     
/*     */   }
/*     */   
/*     */   private static Symbol mkEnumAdjust(List<String> wsymbols, List<String> rsymbols)
/*     */   {
/* 406 */     Object[] adjustments = new Object[wsymbols.size()];
/* 407 */     for (int i = 0; i < adjustments.length; i++) {
/* 408 */       int j = rsymbols.indexOf(wsymbols.get(i));
/* 409 */       adjustments[i] = (j == -1 ? "No match for " + (String)wsymbols.get(i) : new Integer(j));
/*     */     }
/*     */     
/* 412 */     return new Symbol.EnumAdjustAction(rsymbols.size(), adjustments);
/*     */   }
/*     */   
/*     */   private static int bestBranch(Schema r, Schema w) {
/* 416 */     Schema.Type vt = w.getType();
/*     */     
/* 418 */     int j = 0;
/* 419 */     for (Schema b : r.getTypes()) {
/* 420 */       if (vt == b.getType())
/* 421 */         if ((vt == Schema.Type.RECORD) || (vt == Schema.Type.ENUM) || (vt == Schema.Type.FIXED))
/*     */         {
/* 423 */           String vname = w.getFullName();
/* 424 */           String bname = b.getFullName();
/* 425 */           if (((vname != null) && (vname.equals(bname))) || ((vname == bname) && (vt == Schema.Type.RECORD)))
/*     */           {
/* 427 */             return j; }
/*     */         } else {
/* 429 */           return j; }
/* 430 */       j++;
/*     */     }
/*     */     
/*     */ 
/* 434 */     j = 0;
/* 435 */     for (Schema b : r.getTypes()) {
/* 436 */       switch (vt) {
/*     */       case INT: 
/* 438 */         switch (b.getType()) {
/*     */         case LONG: case DOUBLE: 
/* 440 */           return j;
/*     */         }
/* 442 */         break;
/*     */       case LONG: 
/*     */       case FLOAT: 
/* 445 */         switch (b.getType()) {
/*     */         case DOUBLE: 
/* 447 */           return j;
/*     */         }
/*     */         break;
/*     */       }
/* 451 */       j++;
/*     */     }
/* 453 */     return -1;
/*     */   }
/*     */   
/*     */ 
/*     */   static class LitS2
/*     */     extends ValidatingGrammarGenerator.LitS
/*     */   {
/*     */     public Schema expected;
/*     */     
/*     */     public LitS2(Schema actual, Schema expected)
/*     */     {
/* 464 */       super();
/* 465 */       this.expected = expected;
/*     */     }
/*     */     
/* 468 */     public boolean equals(Object o) { if (!(o instanceof LitS2)) return false;
/* 469 */       LitS2 other = (LitS2)o;
/* 470 */       return (this.actual == other.actual) && (this.expected == other.expected);
/*     */     }
/*     */     
/* 473 */     public int hashCode() { return super.hashCode() + this.expected.hashCode(); }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\io\parsing\ResolvingGrammarGenerator.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */