/*     */ package org.codehaus.jackson.map.deser;
/*     */ 
/*     */ import java.util.Collection;
/*     */ import java.util.HashMap;
/*     */ import java.util.Map;
/*     */ import org.codehaus.jackson.JsonNode;
/*     */ import org.codehaus.jackson.map.AnnotationIntrospector;
/*     */ import org.codehaus.jackson.map.BeanProperty;
/*     */ import org.codehaus.jackson.map.BeanProperty.Std;
/*     */ import org.codehaus.jackson.map.ContextualDeserializer;
/*     */ import org.codehaus.jackson.map.DeserializationConfig;
/*     */ import org.codehaus.jackson.map.DeserializationConfig.Feature;
/*     */ import org.codehaus.jackson.map.DeserializerProvider;
/*     */ import org.codehaus.jackson.map.JsonDeserializer;
/*     */ import org.codehaus.jackson.map.JsonMappingException;
/*     */ import org.codehaus.jackson.map.KeyDeserializer;
/*     */ import org.codehaus.jackson.map.TypeDeserializer;
/*     */ import org.codehaus.jackson.map.ext.OptionalHandlerFactory;
/*     */ import org.codehaus.jackson.map.introspect.Annotated;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedMember;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedMethod;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedParameter;
/*     */ import org.codehaus.jackson.map.introspect.BasicBeanDescription;
/*     */ import org.codehaus.jackson.map.jsontype.NamedType;
/*     */ import org.codehaus.jackson.map.jsontype.SubtypeResolver;
/*     */ import org.codehaus.jackson.map.jsontype.TypeResolverBuilder;
/*     */ import org.codehaus.jackson.map.type.ArrayType;
/*     */ import org.codehaus.jackson.map.type.CollectionLikeType;
/*     */ import org.codehaus.jackson.map.type.CollectionType;
/*     */ import org.codehaus.jackson.map.type.MapLikeType;
/*     */ import org.codehaus.jackson.map.type.MapType;
/*     */ import org.codehaus.jackson.map.type.TypeFactory;
/*     */ import org.codehaus.jackson.type.JavaType;
/*     */ 
/*     */ public abstract class BasicDeserializerFactory extends org.codehaus.jackson.map.DeserializerFactory
/*     */ {
/*  37 */   static final HashMap<JavaType, JsonDeserializer<Object>> _simpleDeserializers = ;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  44 */   static final HashMap<String, Class<? extends Map>> _mapFallbacks = new HashMap();
/*     */   static final HashMap<String, Class<? extends Collection>> _collectionFallbacks;
/*     */   
/*  47 */   static { _mapFallbacks.put(Map.class.getName(), java.util.LinkedHashMap.class);
/*  48 */     _mapFallbacks.put(java.util.concurrent.ConcurrentMap.class.getName(), java.util.concurrent.ConcurrentHashMap.class);
/*  49 */     _mapFallbacks.put(java.util.SortedMap.class.getName(), java.util.TreeMap.class);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  56 */     _mapFallbacks.put("java.util.NavigableMap", java.util.TreeMap.class);
/*     */     try {
/*  58 */       Class<?> key = Class.forName("java.util.ConcurrentNavigableMap");
/*  59 */       Class<?> value = Class.forName("java.util.ConcurrentSkipListMap");
/*     */       
/*  61 */       Class<? extends Map> mapValue = value;
/*  62 */       _mapFallbacks.put(key.getName(), mapValue);
/*     */     }
/*     */     catch (ClassNotFoundException cnfe) {}
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  72 */     _collectionFallbacks = new HashMap();
/*     */     
/*     */ 
/*  75 */     _collectionFallbacks.put(Collection.class.getName(), java.util.ArrayList.class);
/*  76 */     _collectionFallbacks.put(java.util.List.class.getName(), java.util.ArrayList.class);
/*  77 */     _collectionFallbacks.put(java.util.Set.class.getName(), java.util.HashSet.class);
/*  78 */     _collectionFallbacks.put(java.util.SortedSet.class.getName(), java.util.TreeSet.class);
/*  79 */     _collectionFallbacks.put(java.util.Queue.class.getName(), java.util.LinkedList.class);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  86 */     _collectionFallbacks.put("java.util.Deque", java.util.LinkedList.class);
/*  87 */     _collectionFallbacks.put("java.util.NavigableSet", java.util.TreeSet.class);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  94 */   protected static final HashMap<JavaType, JsonDeserializer<Object>> _arrayDeserializers = ArrayDeserializers.getAll();
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 100 */   protected OptionalHandlerFactory optionalHandlers = OptionalHandlerFactory.instance;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonDeserializer<?> createArrayDeserializer(DeserializationConfig config, DeserializerProvider p, ArrayType type, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 171 */     JavaType elemType = type.getContentType();
/*     */     
/*     */ 
/* 174 */     JsonDeserializer<Object> contentDeser = (JsonDeserializer)elemType.getValueHandler();
/* 175 */     if (contentDeser == null)
/*     */     {
/* 177 */       JsonDeserializer<?> deser = (JsonDeserializer)_arrayDeserializers.get(elemType);
/* 178 */       if (deser != null)
/*     */       {
/*     */ 
/*     */ 
/*     */ 
/* 183 */         JsonDeserializer<?> custom = _findCustomArrayDeserializer(type, config, p, property, null, null);
/* 184 */         if (custom != null) {
/* 185 */           return custom;
/*     */         }
/* 187 */         return deser;
/*     */       }
/*     */       
/* 190 */       if (elemType.isPrimitive()) {
/* 191 */         throw new IllegalArgumentException("Internal error: primitive type (" + type + ") passed, no array deserializer found");
/*     */       }
/*     */     }
/*     */     
/* 195 */     TypeDeserializer elemTypeDeser = (TypeDeserializer)elemType.getTypeHandler();
/*     */     
/* 197 */     if (elemTypeDeser == null) {
/* 198 */       elemTypeDeser = findTypeDeserializer(config, elemType, property);
/*     */     }
/*     */     
/* 201 */     JsonDeserializer<?> custom = _findCustomArrayDeserializer(type, config, p, property, elemTypeDeser, contentDeser);
/* 202 */     if (custom != null) {
/* 203 */       return custom;
/*     */     }
/*     */     
/* 206 */     if (contentDeser == null)
/*     */     {
/* 208 */       contentDeser = p.findValueDeserializer(config, elemType, property);
/*     */     }
/* 210 */     return new ArrayDeserializer(type, contentDeser, elemTypeDeser);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonDeserializer<?> createCollectionDeserializer(DeserializationConfig config, DeserializerProvider p, CollectionType type, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 219 */     type = (CollectionType)mapAbstractType(config, type);
/*     */     
/* 221 */     Class<?> collectionClass = type.getRawClass();
/* 222 */     BasicBeanDescription beanDesc = (BasicBeanDescription)config.introspectClassAnnotations(collectionClass);
/*     */     
/* 224 */     JsonDeserializer<Object> deser = findDeserializerFromAnnotation(config, beanDesc.getClassInfo(), property);
/* 225 */     if (deser != null) {
/* 226 */       return deser;
/*     */     }
/*     */     
/* 229 */     type = (CollectionType)modifyTypeByAnnotation(config, beanDesc.getClassInfo(), type, null);
/*     */     
/* 231 */     JavaType contentType = type.getContentType();
/*     */     
/* 233 */     JsonDeserializer<Object> contentDeser = (JsonDeserializer)contentType.getValueHandler();
/*     */     
/*     */ 
/* 236 */     TypeDeserializer contentTypeDeser = (TypeDeserializer)contentType.getTypeHandler();
/*     */     
/* 238 */     if (contentTypeDeser == null) {
/* 239 */       contentTypeDeser = findTypeDeserializer(config, contentType, property);
/*     */     }
/*     */     
/*     */ 
/* 243 */     JsonDeserializer<?> custom = _findCustomCollectionDeserializer(type, config, p, beanDesc, property, contentTypeDeser, contentDeser);
/*     */     
/* 245 */     if (custom != null) {
/* 246 */       return custom;
/*     */     }
/*     */     
/* 249 */     if (contentDeser == null)
/*     */     {
/* 251 */       if (java.util.EnumSet.class.isAssignableFrom(collectionClass)) {
/* 252 */         return new EnumSetDeserializer(contentType.getRawClass(), createEnumDeserializer(config, p, contentType, property));
/*     */       }
/*     */       
/*     */ 
/*     */ 
/* 257 */       contentDeser = p.findValueDeserializer(config, contentType, property);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 269 */     if ((type.isInterface()) || (type.isAbstract()))
/*     */     {
/* 271 */       Class<? extends Collection> fallback = (Class)_collectionFallbacks.get(collectionClass.getName());
/* 272 */       if (fallback == null) {
/* 273 */         throw new IllegalArgumentException("Can not find a deserializer for non-concrete Collection type " + type);
/*     */       }
/* 275 */       collectionClass = fallback;
/*     */     }
/*     */     
/* 278 */     boolean fixAccess = config.isEnabled(DeserializationConfig.Feature.CAN_OVERRIDE_ACCESS_MODIFIERS);
/*     */     
/* 280 */     java.lang.reflect.Constructor<Collection<Object>> ctor = org.codehaus.jackson.map.util.ClassUtil.findConstructor(collectionClass, fixAccess);
/*     */     
/* 282 */     if (contentType.getRawClass() == String.class)
/*     */     {
/* 284 */       return new org.codehaus.jackson.map.deser.impl.StringCollectionDeserializer(type, contentDeser, ctor);
/*     */     }
/* 286 */     return new CollectionDeserializer(type, contentDeser, contentTypeDeser, ctor);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonDeserializer<?> createCollectionLikeDeserializer(DeserializationConfig config, DeserializerProvider p, CollectionLikeType type, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 296 */     type = (CollectionLikeType)mapAbstractType(config, type);
/*     */     
/* 298 */     Class<?> collectionClass = type.getRawClass();
/* 299 */     BasicBeanDescription beanDesc = (BasicBeanDescription)config.introspectClassAnnotations(collectionClass);
/*     */     
/* 301 */     JsonDeserializer<Object> deser = findDeserializerFromAnnotation(config, beanDesc.getClassInfo(), property);
/* 302 */     if (deser != null) {
/* 303 */       return deser;
/*     */     }
/*     */     
/* 306 */     type = (CollectionLikeType)modifyTypeByAnnotation(config, beanDesc.getClassInfo(), type, null);
/*     */     
/* 308 */     JavaType contentType = type.getContentType();
/*     */     
/* 310 */     JsonDeserializer<Object> contentDeser = (JsonDeserializer)contentType.getValueHandler();
/*     */     
/*     */ 
/* 313 */     TypeDeserializer contentTypeDeser = (TypeDeserializer)contentType.getTypeHandler();
/*     */     
/* 315 */     if (contentTypeDeser == null) {
/* 316 */       contentTypeDeser = findTypeDeserializer(config, contentType, property);
/*     */     }
/* 318 */     return _findCustomCollectionLikeDeserializer(type, config, p, beanDesc, property, contentTypeDeser, contentDeser);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonDeserializer<?> createMapDeserializer(DeserializationConfig config, DeserializerProvider p, MapType type, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 328 */     type = (MapType)mapAbstractType(config, type);
/*     */     
/* 330 */     BasicBeanDescription beanDesc = (BasicBeanDescription)config.introspectForCreation(type);
/*     */     
/* 332 */     JsonDeserializer<Object> deser = findDeserializerFromAnnotation(config, beanDesc.getClassInfo(), property);
/* 333 */     if (deser != null) {
/* 334 */       return deser;
/*     */     }
/*     */     
/* 337 */     type = (MapType)modifyTypeByAnnotation(config, beanDesc.getClassInfo(), type, null);
/* 338 */     JavaType keyType = type.getKeyType();
/* 339 */     JavaType contentType = type.getContentType();
/*     */     
/*     */ 
/*     */ 
/* 343 */     JsonDeserializer<Object> contentDeser = (JsonDeserializer)contentType.getValueHandler();
/*     */     
/*     */ 
/* 346 */     KeyDeserializer keyDes = (KeyDeserializer)keyType.getValueHandler();
/* 347 */     if (keyDes == null) {
/* 348 */       keyDes = p.findKeyDeserializer(config, keyType, property);
/*     */     }
/*     */     
/* 351 */     TypeDeserializer contentTypeDeser = (TypeDeserializer)contentType.getTypeHandler();
/*     */     
/* 353 */     if (contentTypeDeser == null) {
/* 354 */       contentTypeDeser = findTypeDeserializer(config, contentType, property);
/*     */     }
/*     */     
/*     */ 
/* 358 */     JsonDeserializer<?> custom = _findCustomMapDeserializer(type, config, p, beanDesc, property, keyDes, contentTypeDeser, contentDeser);
/*     */     
/*     */ 
/* 361 */     if (custom != null) {
/* 362 */       return custom;
/*     */     }
/*     */     
/* 365 */     if (contentDeser == null)
/*     */     {
/* 367 */       contentDeser = p.findValueDeserializer(config, contentType, property);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 372 */     Class<?> mapClass = type.getRawClass();
/* 373 */     if (java.util.EnumMap.class.isAssignableFrom(mapClass)) {
/* 374 */       Class<?> kt = keyType.getRawClass();
/* 375 */       if ((kt == null) || (!kt.isEnum())) {
/* 376 */         throw new IllegalArgumentException("Can not construct EnumMap; generic (key) type not available");
/*     */       }
/* 378 */       return new EnumMapDeserializer(keyType.getRawClass(), createEnumDeserializer(config, p, keyType, property), contentDeser);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 394 */     if ((type.isInterface()) || (type.isAbstract()))
/*     */     {
/* 396 */       Class<? extends Map> fallback = (Class)_mapFallbacks.get(mapClass.getName());
/* 397 */       if (fallback == null) {
/* 398 */         throw new IllegalArgumentException("Can not find a deserializer for non-concrete Map type " + type);
/*     */       }
/* 400 */       mapClass = fallback;
/* 401 */       type = (MapType)type.forcedNarrowBy(mapClass);
/*     */       
/* 403 */       beanDesc = (BasicBeanDescription)config.introspectForCreation(type);
/*     */     }
/*     */     
/*     */ 
/* 407 */     boolean fixAccess = config.isEnabled(DeserializationConfig.Feature.CAN_OVERRIDE_ACCESS_MODIFIERS);
/*     */     
/*     */ 
/* 410 */     java.lang.reflect.Constructor<Map<Object, Object>> defaultCtor = beanDesc.findDefaultConstructor();
/* 411 */     if ((defaultCtor != null) && 
/* 412 */       (fixAccess)) {
/* 413 */       org.codehaus.jackson.map.util.ClassUtil.checkAndFixAccess(defaultCtor);
/*     */     }
/*     */     
/* 416 */     MapDeserializer md = new MapDeserializer(type, defaultCtor, keyDes, contentDeser, contentTypeDeser);
/* 417 */     md.setIgnorableProperties(config.getAnnotationIntrospector().findPropertiesToIgnore(beanDesc.getClassInfo()));
/* 418 */     md.setCreators(findMapCreators(config, beanDesc));
/* 419 */     return md;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonDeserializer<?> createMapLikeDeserializer(DeserializationConfig config, DeserializerProvider p, MapLikeType type, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 429 */     type = (MapLikeType)mapAbstractType(config, type);
/* 430 */     BasicBeanDescription beanDesc = (BasicBeanDescription)config.introspectForCreation(type);
/*     */     
/* 432 */     JsonDeserializer<Object> deser = findDeserializerFromAnnotation(config, beanDesc.getClassInfo(), property);
/* 433 */     if (deser != null) {
/* 434 */       return deser;
/*     */     }
/*     */     
/* 437 */     type = (MapLikeType)modifyTypeByAnnotation(config, beanDesc.getClassInfo(), type, null);
/* 438 */     JavaType keyType = type.getKeyType();
/* 439 */     JavaType contentType = type.getContentType();
/*     */     
/*     */ 
/*     */ 
/* 443 */     JsonDeserializer<Object> contentDeser = (JsonDeserializer)contentType.getValueHandler();
/*     */     
/*     */ 
/* 446 */     KeyDeserializer keyDes = (KeyDeserializer)keyType.getValueHandler();
/* 447 */     if (keyDes == null) {
/* 448 */       keyDes = p.findKeyDeserializer(config, keyType, property);
/*     */     }
/*     */     
/* 451 */     TypeDeserializer contentTypeDeser = (TypeDeserializer)contentType.getTypeHandler();
/*     */     
/* 453 */     if (contentTypeDeser == null) {
/* 454 */       contentTypeDeser = findTypeDeserializer(config, contentType, property);
/*     */     }
/* 456 */     return _findCustomMapLikeDeserializer(type, config, p, beanDesc, property, keyDes, contentTypeDeser, contentDeser);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonDeserializer<?> createEnumDeserializer(DeserializationConfig config, DeserializerProvider p, JavaType type, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 471 */     BasicBeanDescription beanDesc = (BasicBeanDescription)config.introspectForCreation(type);
/* 472 */     JsonDeserializer<?> des = findDeserializerFromAnnotation(config, beanDesc.getClassInfo(), property);
/* 473 */     if (des != null) {
/* 474 */       return des;
/*     */     }
/* 476 */     Class<?> enumClass = type.getRawClass();
/*     */     
/* 478 */     JsonDeserializer<?> custom = _findCustomEnumDeserializer(enumClass, config, beanDesc, property);
/* 479 */     if (custom != null) {
/* 480 */       return custom;
/*     */     }
/*     */     
/*     */ 
/* 484 */     for (AnnotatedMethod factory : beanDesc.getFactoryMethods()) {
/* 485 */       if (config.getAnnotationIntrospector().hasCreatorAnnotation(factory)) {
/* 486 */         int argCount = factory.getParameterCount();
/* 487 */         if (argCount == 1) {
/* 488 */           Class<?> returnType = factory.getRawType();
/*     */           
/* 490 */           if (returnType.isAssignableFrom(enumClass)) {
/* 491 */             return EnumDeserializer.deserializerForCreator(config, enumClass, factory);
/*     */           }
/*     */         }
/* 494 */         throw new IllegalArgumentException("Unsuitable method (" + factory + ") decorated with @JsonCreator (for Enum type " + enumClass.getName() + ")");
/*     */       }
/*     */     }
/*     */     
/* 498 */     return new EnumDeserializer(constructEnumResolver(enumClass, config));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonDeserializer<?> createTreeDeserializer(DeserializationConfig config, DeserializerProvider p, JavaType nodeType, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 507 */     Class<? extends JsonNode> nodeClass = nodeType.getRawClass();
/*     */     
/* 509 */     JsonDeserializer<?> custom = _findCustomTreeNodeDeserializer(nodeClass, config, property);
/* 510 */     if (custom != null) {
/* 511 */       return custom;
/*     */     }
/* 513 */     return JsonNodeDeserializer.getDeserializer(nodeClass);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonDeserializer<Object> findStdBeanDeserializer(DeserializationConfig config, DeserializerProvider p, JavaType type, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 528 */     JsonDeserializer<Object> deser = (JsonDeserializer)_simpleDeserializers.get(type);
/* 529 */     if (deser != null) {
/* 530 */       return deser;
/*     */     }
/* 532 */     Class<?> cls = type.getRawClass();
/*     */     
/* 534 */     if (cls == Class.class) {
/* 535 */       return new StdDeserializer.ClassDeserializer();
/*     */     }
/*     */     
/*     */ 
/* 539 */     if (java.util.concurrent.atomic.AtomicReference.class.isAssignableFrom(cls))
/*     */     {
/* 541 */       TypeFactory tf = config.getTypeFactory();
/* 542 */       JavaType[] params = tf.findTypeParameters(type, java.util.concurrent.atomic.AtomicReference.class);
/*     */       JavaType referencedType;
/* 544 */       JavaType referencedType; if ((params == null) || (params.length < 1)) {
/* 545 */         referencedType = TypeFactory.unknownType();
/*     */       } else {
/* 547 */         referencedType = params[0];
/*     */       }
/*     */       
/* 550 */       JsonDeserializer<?> d2 = new StdDeserializer.AtomicReferenceDeserializer(referencedType, property);
/* 551 */       return d2;
/*     */     }
/*     */     
/* 554 */     JsonDeserializer<?> d = this.optionalHandlers.findDeserializer(type, config, p);
/* 555 */     if (d != null) {
/* 556 */       return d;
/*     */     }
/* 558 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public TypeDeserializer findTypeDeserializer(DeserializationConfig config, JavaType baseType, BeanProperty property)
/*     */   {
/* 565 */     Class<?> cls = baseType.getRawClass();
/* 566 */     BasicBeanDescription bean = (BasicBeanDescription)config.introspectClassAnnotations(cls);
/* 567 */     org.codehaus.jackson.map.introspect.AnnotatedClass ac = bean.getClassInfo();
/* 568 */     AnnotationIntrospector ai = config.getAnnotationIntrospector();
/* 569 */     TypeResolverBuilder<?> b = ai.findTypeResolver(config, ac, baseType);
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 574 */     Collection<NamedType> subtypes = null;
/* 575 */     if (b == null) {
/* 576 */       b = config.getDefaultTyper(baseType);
/* 577 */       if (b == null) {
/* 578 */         return null;
/*     */       }
/*     */     } else {
/* 581 */       subtypes = config.getSubtypeResolver().collectAndResolveSubtypes(ac, config, ai);
/*     */     }
/* 583 */     return b.buildTypeDeserializer(config, baseType, subtypes, property);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public TypeDeserializer findPropertyTypeDeserializer(DeserializationConfig config, JavaType baseType, AnnotatedMember annotated, BeanProperty property)
/*     */   {
/* 610 */     AnnotationIntrospector ai = config.getAnnotationIntrospector();
/* 611 */     TypeResolverBuilder<?> b = ai.findPropertyTypeResolver(config, annotated, baseType);
/*     */     
/*     */ 
/* 614 */     if (b == null) {
/* 615 */       return findTypeDeserializer(config, baseType, property);
/*     */     }
/*     */     
/* 618 */     Collection<NamedType> subtypes = config.getSubtypeResolver().collectAndResolveSubtypes(annotated, config, ai);
/* 619 */     return b.buildTypeDeserializer(config, baseType, subtypes, property);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public TypeDeserializer findPropertyContentTypeDeserializer(DeserializationConfig config, JavaType containerType, AnnotatedMember propertyEntity, BeanProperty property)
/*     */   {
/* 638 */     AnnotationIntrospector ai = config.getAnnotationIntrospector();
/* 639 */     TypeResolverBuilder<?> b = ai.findPropertyContentTypeResolver(config, propertyEntity, containerType);
/* 640 */     JavaType contentType = containerType.getContentType();
/*     */     
/* 642 */     if (b == null) {
/* 643 */       return findTypeDeserializer(config, contentType, property);
/*     */     }
/*     */     
/* 646 */     Collection<NamedType> subtypes = config.getSubtypeResolver().collectAndResolveSubtypes(propertyEntity, config, ai);
/* 647 */     return b.buildTypeDeserializer(config, contentType, subtypes, property);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonDeserializer<Object> findDeserializerFromAnnotation(DeserializationConfig config, Annotated ann, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 668 */     Object deserDef = config.getAnnotationIntrospector().findDeserializer(ann);
/* 669 */     if (deserDef != null) {
/* 670 */       return _constructDeserializer(config, ann, property, deserDef);
/*     */     }
/* 672 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   JsonDeserializer<Object> _constructDeserializer(DeserializationConfig config, Annotated ann, BeanProperty property, Object deserDef)
/*     */     throws JsonMappingException
/*     */   {
/* 680 */     if ((deserDef instanceof JsonDeserializer)) {
/* 681 */       JsonDeserializer<Object> deser = (JsonDeserializer)deserDef;
/*     */       
/* 683 */       if ((deser instanceof ContextualDeserializer)) {
/* 684 */         deser = ((ContextualDeserializer)deser).createContextual(config, property);
/*     */       }
/* 686 */       return deser;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 691 */     if (!(deserDef instanceof Class)) {
/* 692 */       throw new IllegalStateException("AnnotationIntrospector returned deserializer definition of type " + deserDef.getClass().getName() + "; expected type JsonDeserializer or Class<JsonDeserializer> instead");
/*     */     }
/* 694 */     Class<? extends JsonDeserializer<?>> deserClass = (Class)deserDef;
/* 695 */     if (!JsonDeserializer.class.isAssignableFrom(deserClass)) {
/* 696 */       throw new IllegalStateException("AnnotationIntrospector returned Class " + deserClass.getName() + "; expected Class<JsonDeserializer>");
/*     */     }
/* 698 */     JsonDeserializer<Object> deser = config.deserializerInstance(ann, deserClass);
/*     */     
/* 700 */     if ((deser instanceof ContextualDeserializer)) {
/* 701 */       deser = ((ContextualDeserializer)deser).createContextual(config, property);
/*     */     }
/* 703 */     return deser;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected <T extends JavaType> T modifyTypeByAnnotation(DeserializationConfig config, Annotated a, T type, String propName)
/*     */     throws JsonMappingException
/*     */   {
/* 731 */     AnnotationIntrospector intr = config.getAnnotationIntrospector();
/* 732 */     Class<?> subclass = intr.findDeserializationType(a, type, propName);
/* 733 */     if (subclass != null) {
/*     */       try {
/* 735 */         type = type.narrowBy(subclass);
/*     */       } catch (IllegalArgumentException iae) {
/* 737 */         throw new JsonMappingException("Failed to narrow type " + type + " with concrete-type annotation (value " + subclass.getName() + "), method '" + a.getName() + "': " + iae.getMessage(), null, iae);
/*     */       }
/*     */     }
/*     */     
/*     */ 
/* 742 */     if (type.isContainerType()) {
/* 743 */       Class<?> keyClass = intr.findDeserializationKeyType(a, type.getKeyType(), propName);
/* 744 */       if (keyClass != null)
/*     */       {
/* 746 */         if (!(type instanceof MapType)) {
/* 747 */           throw new JsonMappingException("Illegal key-type annotation: type " + type + " is not a Map type");
/*     */         }
/*     */         try {
/* 750 */           type = ((MapType)type).narrowKey(keyClass);
/*     */         } catch (IllegalArgumentException iae) {
/* 752 */           throw new JsonMappingException("Failed to narrow key type " + type + " with key-type annotation (" + keyClass.getName() + "): " + iae.getMessage(), null, iae);
/*     */         }
/*     */       }
/* 755 */       JavaType keyType = type.getKeyType();
/*     */       
/*     */ 
/*     */ 
/*     */ 
/* 760 */       if ((keyType != null) && (keyType.getValueHandler() == null)) {
/* 761 */         Class<? extends KeyDeserializer> kdClass = intr.findKeyDeserializer(a);
/* 762 */         if ((kdClass != null) && (kdClass != org.codehaus.jackson.map.KeyDeserializer.None.class)) {
/* 763 */           KeyDeserializer kd = config.keyDeserializerInstance(a, kdClass);
/* 764 */           keyType.setValueHandler(kd);
/*     */         }
/*     */       }
/*     */       
/*     */ 
/* 769 */       Class<?> cc = intr.findDeserializationContentType(a, type.getContentType(), propName);
/* 770 */       if (cc != null) {
/*     */         try {
/* 772 */           type = type.narrowContentsBy(cc);
/*     */         } catch (IllegalArgumentException iae) {
/* 774 */           throw new JsonMappingException("Failed to narrow content type " + type + " with content-type annotation (" + cc.getName() + "): " + iae.getMessage(), null, iae);
/*     */         }
/*     */       }
/*     */       
/* 778 */       JavaType contentType = type.getContentType();
/* 779 */       if (contentType.getValueHandler() == null) {
/* 780 */         Class<? extends JsonDeserializer<?>> cdClass = intr.findContentDeserializer(a);
/* 781 */         if ((cdClass != null) && (cdClass != org.codehaus.jackson.map.JsonDeserializer.None.class)) {
/* 782 */           JsonDeserializer<Object> cd = config.deserializerInstance(a, cdClass);
/* 783 */           type.getContentType().setValueHandler(cd);
/*     */         }
/*     */       }
/*     */     }
/* 787 */     return type;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JavaType resolveType(DeserializationConfig config, BasicBeanDescription beanDesc, JavaType type, AnnotatedMember member, BeanProperty property)
/*     */   {
/* 805 */     if (type.isContainerType()) {
/* 806 */       AnnotationIntrospector intr = config.getAnnotationIntrospector();
/* 807 */       JavaType keyType = type.getKeyType();
/* 808 */       if (keyType != null) {
/* 809 */         Class<? extends KeyDeserializer> kdClass = intr.findKeyDeserializer(member);
/* 810 */         if ((kdClass != null) && (kdClass != org.codehaus.jackson.map.KeyDeserializer.None.class)) {
/* 811 */           KeyDeserializer kd = config.keyDeserializerInstance(member, kdClass);
/* 812 */           keyType.setValueHandler(kd);
/*     */         }
/*     */       }
/*     */       
/* 816 */       Class<? extends JsonDeserializer<?>> cdClass = intr.findContentDeserializer(member);
/* 817 */       if ((cdClass != null) && (cdClass != org.codehaus.jackson.map.JsonDeserializer.None.class)) {
/* 818 */         JsonDeserializer<Object> cd = config.deserializerInstance(member, cdClass);
/* 819 */         type.getContentType().setValueHandler(cd);
/*     */       }
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 826 */       if ((member instanceof AnnotatedMember)) {
/* 827 */         TypeDeserializer contentTypeDeser = findPropertyContentTypeDeserializer(config, type, member, property);
/*     */         
/* 829 */         if (contentTypeDeser != null) {
/* 830 */           type = type.withContentTypeHandler(contentTypeDeser);
/*     */         }
/*     */       }
/*     */     }
/*     */     TypeDeserializer valueTypeDeser;
/*     */     TypeDeserializer valueTypeDeser;
/* 836 */     if ((member instanceof AnnotatedMember)) {
/* 837 */       valueTypeDeser = findPropertyTypeDeserializer(config, type, member, property);
/*     */     }
/*     */     else {
/* 840 */       valueTypeDeser = findTypeDeserializer(config, type, null);
/*     */     }
/* 842 */     if (valueTypeDeser != null) {
/* 843 */       type = type.withTypeHandler(valueTypeDeser);
/*     */     }
/* 845 */     return type;
/*     */   }
/*     */   
/*     */ 
/*     */   protected EnumResolver<?> constructEnumResolver(Class<?> enumClass, DeserializationConfig config)
/*     */   {
/* 851 */     if (config.isEnabled(DeserializationConfig.Feature.READ_ENUMS_USING_TO_STRING)) {
/* 852 */       return EnumResolver.constructUnsafeUsingToString(enumClass);
/*     */     }
/* 854 */     return EnumResolver.constructUnsafe(enumClass, config.getAnnotationIntrospector());
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected CreatorContainer findMapCreators(DeserializationConfig config, BasicBeanDescription beanDesc)
/*     */     throws JsonMappingException
/*     */   {
/* 870 */     AnnotationIntrospector intr = config.getAnnotationIntrospector();
/* 871 */     boolean fixAccess = config.isEnabled(DeserializationConfig.Feature.CAN_OVERRIDE_ACCESS_MODIFIERS);
/* 872 */     CreatorContainer creators = new CreatorContainer(beanDesc, fixAccess);
/*     */     
/* 874 */     for (org.codehaus.jackson.map.introspect.AnnotatedConstructor ctor : beanDesc.getConstructors()) {
/* 875 */       int argCount = ctor.getParameterCount();
/* 876 */       if ((argCount >= 1) && (intr.hasCreatorAnnotation(ctor)))
/*     */       {
/*     */ 
/*     */ 
/* 880 */         SettableBeanProperty[] properties = new SettableBeanProperty[argCount];
/* 881 */         int nameCount = 0;
/* 882 */         for (int i = 0; i < argCount; i++) {
/* 883 */           AnnotatedParameter param = ctor.getParameter(i);
/* 884 */           String name = param == null ? null : intr.findPropertyNameForParam(param);
/*     */           
/* 886 */           if ((name == null) || (name.length() == 0)) {
/* 887 */             throw new IllegalArgumentException("Parameter #" + i + " of constructor " + ctor + " has no property name annotation: must have for @JsonCreator for a Map type");
/*     */           }
/* 889 */           nameCount++;
/* 890 */           properties[i] = constructCreatorProperty(config, beanDesc, name, i, param);
/*     */         }
/* 892 */         creators.addPropertyConstructor(ctor, properties);
/*     */       }
/*     */     }
/*     */     
/* 896 */     for (AnnotatedMethod factory : beanDesc.getFactoryMethods()) {
/* 897 */       int argCount = factory.getParameterCount();
/* 898 */       if ((argCount >= 1) && (intr.hasCreatorAnnotation(factory)))
/*     */       {
/*     */ 
/*     */ 
/* 902 */         SettableBeanProperty[] properties = new SettableBeanProperty[argCount];
/* 903 */         int nameCount = 0;
/* 904 */         for (int i = 0; i < argCount; i++) {
/* 905 */           AnnotatedParameter param = factory.getParameter(i);
/* 906 */           String name = param == null ? null : intr.findPropertyNameForParam(param);
/*     */           
/* 908 */           if ((name == null) || (name.length() == 0)) {
/* 909 */             throw new IllegalArgumentException("Parameter #" + i + " of factory method " + factory + " has no property name annotation: must have for @JsonCreator for a Map type");
/*     */           }
/* 911 */           nameCount++;
/* 912 */           properties[i] = constructCreatorProperty(config, beanDesc, name, i, param);
/*     */         }
/* 914 */         creators.addPropertyFactory(factory, properties);
/*     */       } }
/* 916 */     return creators;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected SettableBeanProperty constructCreatorProperty(DeserializationConfig config, BasicBeanDescription beanDesc, String name, int index, AnnotatedParameter param)
/*     */     throws JsonMappingException
/*     */   {
/* 929 */     JavaType t0 = config.getTypeFactory().constructType(param.getParameterType(), beanDesc.bindingsForBeanType());
/* 930 */     BeanProperty.Std property = new BeanProperty.Std(name, t0, beanDesc.getClassAnnotations(), param);
/* 931 */     JavaType type = resolveType(config, beanDesc, t0, param, property);
/* 932 */     if (type != t0) {
/* 933 */       property = property.withType(type);
/*     */     }
/*     */     
/* 936 */     JsonDeserializer<Object> deser = findDeserializerFromAnnotation(config, param, property);
/*     */     
/* 938 */     type = modifyTypeByAnnotation(config, param, type, name);
/* 939 */     TypeDeserializer typeDeser = findTypeDeserializer(config, type, property);
/* 940 */     SettableBeanProperty prop = new SettableBeanProperty.CreatorProperty(name, type, typeDeser, beanDesc.getClassAnnotations(), param, index);
/*     */     
/* 942 */     if (deser != null) {
/* 943 */       prop.setValueDeserializer(deser);
/*     */     }
/* 945 */     return prop;
/*     */   }
/*     */   
/*     */   public abstract org.codehaus.jackson.map.DeserializerFactory withConfig(org.codehaus.jackson.map.DeserializerFactory.Config paramConfig);
/*     */   
/*     */   protected abstract JsonDeserializer<?> _findCustomArrayDeserializer(ArrayType paramArrayType, DeserializationConfig paramDeserializationConfig, DeserializerProvider paramDeserializerProvider, BeanProperty paramBeanProperty, TypeDeserializer paramTypeDeserializer, JsonDeserializer<?> paramJsonDeserializer)
/*     */     throws JsonMappingException;
/*     */   
/*     */   protected abstract JsonDeserializer<?> _findCustomCollectionDeserializer(CollectionType paramCollectionType, DeserializationConfig paramDeserializationConfig, DeserializerProvider paramDeserializerProvider, BasicBeanDescription paramBasicBeanDescription, BeanProperty paramBeanProperty, TypeDeserializer paramTypeDeserializer, JsonDeserializer<?> paramJsonDeserializer)
/*     */     throws JsonMappingException;
/*     */   
/*     */   protected abstract JsonDeserializer<?> _findCustomCollectionLikeDeserializer(CollectionLikeType paramCollectionLikeType, DeserializationConfig paramDeserializationConfig, DeserializerProvider paramDeserializerProvider, BasicBeanDescription paramBasicBeanDescription, BeanProperty paramBeanProperty, TypeDeserializer paramTypeDeserializer, JsonDeserializer<?> paramJsonDeserializer)
/*     */     throws JsonMappingException;
/*     */   
/*     */   protected abstract JsonDeserializer<?> _findCustomEnumDeserializer(Class<?> paramClass, DeserializationConfig paramDeserializationConfig, BasicBeanDescription paramBasicBeanDescription, BeanProperty paramBeanProperty)
/*     */     throws JsonMappingException;
/*     */   
/*     */   protected abstract JsonDeserializer<?> _findCustomMapDeserializer(MapType paramMapType, DeserializationConfig paramDeserializationConfig, DeserializerProvider paramDeserializerProvider, BasicBeanDescription paramBasicBeanDescription, BeanProperty paramBeanProperty, KeyDeserializer paramKeyDeserializer, TypeDeserializer paramTypeDeserializer, JsonDeserializer<?> paramJsonDeserializer)
/*     */     throws JsonMappingException;
/*     */   
/*     */   protected abstract JsonDeserializer<?> _findCustomMapLikeDeserializer(MapLikeType paramMapLikeType, DeserializationConfig paramDeserializationConfig, DeserializerProvider paramDeserializerProvider, BasicBeanDescription paramBasicBeanDescription, BeanProperty paramBeanProperty, KeyDeserializer paramKeyDeserializer, TypeDeserializer paramTypeDeserializer, JsonDeserializer<?> paramJsonDeserializer)
/*     */     throws JsonMappingException;
/*     */   
/*     */   protected abstract JsonDeserializer<?> _findCustomTreeNodeDeserializer(Class<? extends JsonNode> paramClass, DeserializationConfig paramDeserializationConfig, BeanProperty paramBeanProperty)
/*     */     throws JsonMappingException;
/*     */   
/*     */   protected abstract JavaType mapAbstractType(DeserializationConfig paramDeserializationConfig, JavaType paramJavaType)
/*     */     throws JsonMappingException;
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\deser\BasicDeserializerFactory.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */