/*     */ package org.codehaus.jackson.map.deser;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.lang.annotation.Annotation;
/*     */ import java.lang.reflect.Field;
/*     */ import java.lang.reflect.Method;
/*     */ import java.util.Collection;
/*     */ import java.util.Map;
/*     */ import org.codehaus.jackson.JsonParser;
/*     */ import org.codehaus.jackson.JsonProcessingException;
/*     */ import org.codehaus.jackson.JsonToken;
/*     */ import org.codehaus.jackson.map.BeanProperty;
/*     */ import org.codehaus.jackson.map.DeserializationConfig.Feature;
/*     */ import org.codehaus.jackson.map.DeserializationContext;
/*     */ import org.codehaus.jackson.map.JsonDeserializer;
/*     */ import org.codehaus.jackson.map.JsonMappingException;
/*     */ import org.codehaus.jackson.map.TypeDeserializer;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedField;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedMember;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedMethod;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedParameter;
/*     */ import org.codehaus.jackson.map.util.Annotations;
/*     */ import org.codehaus.jackson.type.JavaType;
/*     */ import org.codehaus.jackson.util.InternCache;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class SettableBeanProperty
/*     */   implements BeanProperty
/*     */ {
/*     */   protected final String _propName;
/*     */   protected final JavaType _type;
/*     */   protected final Annotations _contextAnnotations;
/*     */   protected JsonDeserializer<Object> _valueDeserializer;
/*     */   protected TypeDeserializer _valueTypeDeserializer;
/*     */   protected NullProvider _nullProvider;
/*     */   protected String _managedReferenceName;
/*  81 */   protected int _propertyIndex = -1;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected SettableBeanProperty(String propName, JavaType type, TypeDeserializer typeDeser, Annotations contextAnnotations)
/*     */   {
/*  96 */     if ((propName == null) || (propName.length() == 0)) {
/*  97 */       this._propName = "";
/*     */     } else {
/*  99 */       this._propName = InternCache.instance.intern(propName);
/*     */     }
/* 101 */     this._type = type;
/* 102 */     this._contextAnnotations = contextAnnotations;
/* 103 */     this._valueTypeDeserializer = typeDeser;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected SettableBeanProperty(SettableBeanProperty src)
/*     */   {
/* 113 */     this._propName = src._propName;
/* 114 */     this._type = src._type;
/* 115 */     this._contextAnnotations = src._contextAnnotations;
/* 116 */     this._valueDeserializer = src._valueDeserializer;
/* 117 */     this._valueTypeDeserializer = src._valueTypeDeserializer;
/* 118 */     this._nullProvider = src._nullProvider;
/* 119 */     this._managedReferenceName = src._managedReferenceName;
/* 120 */     this._propertyIndex = src._propertyIndex;
/*     */   }
/*     */   
/*     */   public void setValueDeserializer(JsonDeserializer<Object> deser)
/*     */   {
/* 125 */     if (this._valueDeserializer != null) {
/* 126 */       throw new IllegalStateException("Already had assigned deserializer for property '" + getName() + "' (class " + getDeclaringClass().getName() + ")");
/*     */     }
/* 128 */     this._valueDeserializer = deser;
/* 129 */     Object nvl = this._valueDeserializer.getNullValue();
/* 130 */     this._nullProvider = (nvl == null ? null : new NullProvider(this._type, nvl));
/*     */   }
/*     */   
/*     */   public void setManagedReferenceName(String n) {
/* 134 */     this._managedReferenceName = n;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void assignIndex(int index)
/*     */   {
/* 143 */     if (this._propertyIndex != -1) {
/* 144 */       throw new IllegalStateException("Property '" + getName() + "' already had index (" + this._propertyIndex + "), trying to assign " + index);
/*     */     }
/* 146 */     this._propertyIndex = index;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 155 */   public final String getName() { return this._propName; }
/*     */   
/* 157 */   public JavaType getType() { return this._type; }
/*     */   
/*     */   public abstract <A extends Annotation> A getAnnotation(Class<A> paramClass);
/*     */   
/*     */   public abstract AnnotatedMember getMember();
/*     */   
/*     */   public <A extends Annotation> A getContextAnnotation(Class<A> acls)
/*     */   {
/* 165 */     return this._contextAnnotations.get(acls);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected final Class<?> getDeclaringClass()
/*     */   {
/* 175 */     return getMember().getDeclaringClass();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   @Deprecated
/* 182 */   public String getPropertyName() { return this._propName; }
/*     */   
/* 184 */   public String getManagedReferenceName() { return this._managedReferenceName; }
/*     */   
/* 186 */   public boolean hasValueDeserializer() { return this._valueDeserializer != null; }
/*     */   
/*     */ 
/*     */   public JsonDeserializer<Object> getValueDeserializer()
/*     */   {
/* 191 */     return this._valueDeserializer;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int getCreatorIndex()
/*     */   {
/* 201 */     return -1;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int getPropertyIndex()
/*     */   {
/* 212 */     return this._propertyIndex;
/*     */   }
/*     */   
/*     */   @Deprecated
/*     */   public int getProperytIndex()
/*     */   {
/* 218 */     return this._propertyIndex;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public abstract void deserializeAndSet(JsonParser paramJsonParser, DeserializationContext paramDeserializationContext, Object paramObject)
/*     */     throws IOException, JsonProcessingException;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public abstract void set(Object paramObject1, Object paramObject2)
/*     */     throws IOException;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final Object deserialize(JsonParser jp, DeserializationContext ctxt)
/*     */     throws IOException, JsonProcessingException
/*     */   {
/* 251 */     JsonToken t = jp.getCurrentToken();
/* 252 */     if (t == JsonToken.VALUE_NULL) {
/* 253 */       return this._nullProvider == null ? null : this._nullProvider.nullValue(ctxt);
/*     */     }
/* 255 */     if (this._valueTypeDeserializer != null) {
/* 256 */       return this._valueDeserializer.deserializeWithType(jp, ctxt, this._valueTypeDeserializer);
/*     */     }
/* 258 */     return this._valueDeserializer.deserialize(jp, ctxt);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void _throwAsIOE(Exception e, Object value)
/*     */     throws IOException
/*     */   {
/* 274 */     if ((e instanceof IllegalArgumentException)) {
/* 275 */       String actType = value == null ? "[NULL]" : value.getClass().getName();
/* 276 */       StringBuilder msg = new StringBuilder("Problem deserializing property '").append(getPropertyName());
/* 277 */       msg.append("' (expected type: ").append(getType());
/* 278 */       msg.append("; actual type: ").append(actType).append(")");
/* 279 */       String origMsg = e.getMessage();
/* 280 */       if (origMsg != null) {
/* 281 */         msg.append(", problem: ").append(origMsg);
/*     */       } else {
/* 283 */         msg.append(" (no error message provided)");
/*     */       }
/* 285 */       throw new JsonMappingException(msg.toString(), null, e);
/*     */     }
/* 287 */     _throwAsIOE(e);
/*     */   }
/*     */   
/*     */   protected IOException _throwAsIOE(Exception e)
/*     */     throws IOException
/*     */   {
/* 293 */     if ((e instanceof IOException)) {
/* 294 */       throw ((IOException)e);
/*     */     }
/* 296 */     if ((e instanceof RuntimeException)) {
/* 297 */       throw ((RuntimeException)e);
/*     */     }
/*     */     
/* 300 */     Throwable th = e;
/* 301 */     while (th.getCause() != null) {
/* 302 */       th = th.getCause();
/*     */     }
/* 304 */     throw new JsonMappingException(th.getMessage(), null, th);
/*     */   }
/*     */   
/* 307 */   public String toString() { return "[property '" + getName() + "']"; }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static final class MethodProperty
/*     */     extends SettableBeanProperty
/*     */   {
/*     */     protected final AnnotatedMethod _annotated;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     protected final Method _setter;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public MethodProperty(String name, JavaType type, TypeDeserializer typeDeser, Annotations contextAnnotations, AnnotatedMethod method)
/*     */     {
/* 333 */       super(type, typeDeser, contextAnnotations);
/* 334 */       this._annotated = method;
/* 335 */       this._setter = method.getAnnotated();
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public <A extends Annotation> A getAnnotation(Class<A> acls)
/*     */     {
/* 346 */       return this._annotated.getAnnotation(acls);
/*     */     }
/*     */     
/* 349 */     public AnnotatedMember getMember() { return this._annotated; }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public void deserializeAndSet(JsonParser jp, DeserializationContext ctxt, Object instance)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 362 */       set(instance, deserialize(jp, ctxt));
/*     */     }
/*     */     
/*     */     public final void set(Object instance, Object value)
/*     */       throws IOException
/*     */     {
/*     */       try
/*     */       {
/* 370 */         this._setter.invoke(instance, new Object[] { value });
/*     */       } catch (Exception e) {
/* 372 */         _throwAsIOE(e, value);
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static final class SetterlessProperty
/*     */     extends SettableBeanProperty
/*     */   {
/*     */     protected final AnnotatedMethod _annotated;
/*     */     
/*     */ 
/*     */ 
/*     */     protected final Method _getter;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     public SetterlessProperty(String name, JavaType type, TypeDeserializer typeDeser, Annotations contextAnnotations, AnnotatedMethod method)
/*     */     {
/* 395 */       super(type, typeDeser, contextAnnotations);
/* 396 */       this._annotated = method;
/* 397 */       this._getter = method.getAnnotated();
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public <A extends Annotation> A getAnnotation(Class<A> acls)
/*     */     {
/* 408 */       return this._annotated.getAnnotation(acls);
/*     */     }
/*     */     
/* 411 */     public AnnotatedMember getMember() { return this._annotated; }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public final void deserializeAndSet(JsonParser jp, DeserializationContext ctxt, Object instance)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 424 */       JsonToken t = jp.getCurrentToken();
/* 425 */       if (t == JsonToken.VALUE_NULL) {
/*     */         return;
/*     */       }
/*     */       
/*     */ 
/*     */       Object toModify;
/*     */       
/*     */ 
/*     */       try
/*     */       {
/* 435 */         toModify = this._getter.invoke(instance, new Object[0]);
/*     */       } catch (Exception e) {
/* 437 */         _throwAsIOE(e);
/* 438 */         return;
/*     */       }
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 445 */       if (toModify == null) {
/* 446 */         throw new JsonMappingException("Problem deserializing 'setterless' property '" + getName() + "': get method returned null");
/*     */       }
/* 448 */       this._valueDeserializer.deserialize(jp, ctxt, toModify);
/*     */     }
/*     */     
/*     */ 
/*     */     public final void set(Object instance, Object value)
/*     */       throws IOException
/*     */     {
/* 455 */       throw new UnsupportedOperationException("Should never call 'set' on setterless property");
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public static final class FieldProperty
/*     */     extends SettableBeanProperty
/*     */   {
/*     */     protected final AnnotatedField _annotated;
/*     */     
/*     */ 
/*     */ 
/*     */     protected final Field _field;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     public FieldProperty(String name, JavaType type, TypeDeserializer typeDeser, Annotations contextAnnotations, AnnotatedField field)
/*     */     {
/* 476 */       super(type, typeDeser, contextAnnotations);
/* 477 */       this._annotated = field;
/* 478 */       this._field = field.getAnnotated();
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public <A extends Annotation> A getAnnotation(Class<A> acls)
/*     */     {
/* 489 */       return this._annotated.getAnnotation(acls);
/*     */     }
/*     */     
/* 492 */     public AnnotatedMember getMember() { return this._annotated; }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public void deserializeAndSet(JsonParser jp, DeserializationContext ctxt, Object instance)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 505 */       set(instance, deserialize(jp, ctxt));
/*     */     }
/*     */     
/*     */     public final void set(Object instance, Object value)
/*     */       throws IOException
/*     */     {
/*     */       try
/*     */       {
/* 513 */         this._field.set(instance, value);
/*     */       } catch (Exception e) {
/* 515 */         _throwAsIOE(e, value);
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static final class CreatorProperty
/*     */     extends SettableBeanProperty
/*     */   {
/*     */     protected final AnnotatedParameter _annotated;
/*     */     
/*     */ 
/*     */ 
/*     */     protected final int _index;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     public CreatorProperty(String name, JavaType type, TypeDeserializer typeDeser, Annotations contextAnnotations, AnnotatedParameter param, int index)
/*     */     {
/* 538 */       super(type, typeDeser, contextAnnotations);
/* 539 */       this._annotated = param;
/* 540 */       this._index = index;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public <A extends Annotation> A getAnnotation(Class<A> acls)
/*     */     {
/* 551 */       return this._annotated.getAnnotation(acls);
/*     */     }
/*     */     
/* 554 */     public AnnotatedMember getMember() { return this._annotated; }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public int getCreatorIndex()
/*     */     {
/* 571 */       return this._index;
/*     */     }
/*     */     
/*     */ 
/*     */     public void deserializeAndSet(JsonParser jp, DeserializationContext ctxt, Object instance)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 578 */       set(instance, deserialize(jp, ctxt));
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public void set(Object instance, Object value)
/*     */       throws IOException
/*     */     {}
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static final class ManagedReferenceProperty
/*     */     extends SettableBeanProperty
/*     */   {
/*     */     protected final String _referenceName;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     protected final boolean _isContainer;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     protected final SettableBeanProperty _managedProperty;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     protected final SettableBeanProperty _backProperty;
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */     public ManagedReferenceProperty(String refName, SettableBeanProperty forward, SettableBeanProperty backward, Annotations contextAnnotations, boolean isContainer)
/*     */     {
/* 619 */       super(forward.getType(), forward._valueTypeDeserializer, contextAnnotations);
/*     */       
/* 621 */       this._referenceName = refName;
/* 622 */       this._managedProperty = forward;
/* 623 */       this._backProperty = backward;
/* 624 */       this._isContainer = isContainer;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public <A extends Annotation> A getAnnotation(Class<A> acls)
/*     */     {
/* 635 */       return this._managedProperty.getAnnotation(acls);
/*     */     }
/*     */     
/* 638 */     public AnnotatedMember getMember() { return this._managedProperty.getMember(); }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     public void deserializeAndSet(JsonParser jp, DeserializationContext ctxt, Object instance)
/*     */       throws IOException, JsonProcessingException
/*     */     {
/* 651 */       set(instance, this._managedProperty.deserialize(jp, ctxt));
/*     */     }
/*     */     
/*     */ 
/*     */     public final void set(Object instance, Object value)
/*     */       throws IOException
/*     */     {
/* 658 */       this._managedProperty.set(instance, value);
/*     */       
/*     */ 
/*     */ 
/* 662 */       if (value != null) {
/* 663 */         if (this._isContainer) {
/* 664 */           if ((value instanceof Object[])) {
/* 665 */             for (Object ob : (Object[])value) {
/* 666 */               if (ob != null) {
/* 667 */                 this._backProperty.set(ob, instance);
/*     */               }
/*     */             }
/* 670 */           } else if ((value instanceof Collection)) {
/* 671 */             for (Object ob : (Collection)value) {
/* 672 */               if (ob != null) {
/* 673 */                 this._backProperty.set(ob, instance);
/*     */               }
/*     */             }
/* 676 */           } else if ((value instanceof Map)) {
/* 677 */             for (Object ob : ((Map)value).values()) {
/* 678 */               if (ob != null) {
/* 679 */                 this._backProperty.set(ob, instance);
/*     */               }
/*     */             }
/*     */           } else {
/* 683 */             throw new IllegalStateException("Unsupported container type (" + value.getClass().getName() + ") when resolving reference '" + this._referenceName + "'");
/*     */           }
/*     */         }
/*     */         else {
/* 687 */           this._backProperty.set(value, instance);
/*     */         }
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   protected static final class NullProvider
/*     */   {
/*     */     private final Object _nullValue;
/*     */     
/*     */ 
/*     */     private final boolean _isPrimitive;
/*     */     
/*     */     private final Class<?> _rawType;
/*     */     
/*     */ 
/*     */     protected NullProvider(JavaType type, Object nullValue)
/*     */     {
/* 707 */       this._nullValue = nullValue;
/*     */       
/* 709 */       this._isPrimitive = type.isPrimitive();
/* 710 */       this._rawType = type.getRawClass();
/*     */     }
/*     */     
/*     */     public Object nullValue(DeserializationContext ctxt) throws JsonProcessingException
/*     */     {
/* 715 */       if ((this._isPrimitive) && (ctxt.isEnabled(DeserializationConfig.Feature.FAIL_ON_NULL_FOR_PRIMITIVES))) {
/* 716 */         throw ctxt.mappingException("Can not map JSON null into type " + this._rawType.getName() + " (set DeserializationConfig.Feature.FAIL_ON_NULL_FOR_PRIMITIVES to 'false' to allow)");
/*     */       }
/*     */       
/* 719 */       return this._nullValue;
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\deser\SettableBeanProperty.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */