/*     */ package org.codehaus.janino;
/*     */ 
/*     */ import java.io.File;
/*     */ import java.util.HashMap;
/*     */ import java.util.HashSet;
/*     */ import java.util.Iterator;
/*     */ import java.util.Map;
/*     */ import java.util.Map.Entry;
/*     */ import java.util.Set;
/*     */ import org.codehaus.commons.compiler.AbstractJavaSourceClassLoader;
/*     */ import org.codehaus.commons.compiler.AbstractJavaSourceClassLoader.ProtectionDomainFactory;
/*     */ import org.codehaus.commons.compiler.CompileException;
/*     */ import org.codehaus.janino.util.ClassFile;
/*     */ import org.codehaus.janino.util.resource.DirectoryResourceFinder;
/*     */ import org.codehaus.janino.util.resource.PathResourceFinder;
/*     */ import org.codehaus.janino.util.resource.ResourceFinder;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class JavaSourceClassLoader
/*     */   extends AbstractJavaSourceClassLoader
/*     */ {
/*     */   private static final boolean DEBUG = false;
/*     */   
/*     */   public JavaSourceClassLoader()
/*     */   {
/*  50 */     this(ClassLoader.getSystemClassLoader());
/*     */   }
/*     */   
/*     */   public JavaSourceClassLoader(ClassLoader parentClassLoader) {
/*  54 */     this(parentClassLoader, (File[])null, null);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JavaSourceClassLoader(ClassLoader parentClassLoader, File[] optionalSourcePath, String optionalCharacterEncoding)
/*     */   {
/*  76 */     this(parentClassLoader, optionalSourcePath == null ? new DirectoryResourceFinder(new File(".")) : new PathResourceFinder(optionalSourcePath), optionalCharacterEncoding);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JavaSourceClassLoader(ClassLoader parentClassLoader, ResourceFinder sourceFinder, String optionalCharacterEncoding)
/*     */   {
/* 105 */     super(parentClassLoader);
/*     */     
/* 107 */     this.iClassLoader = new JavaSourceIClassLoader(sourceFinder, optionalCharacterEncoding, this.unitCompilers, new ClassLoaderIClassLoader(parentClassLoader));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void setSourcePath(File[] sourcePath)
/*     */   {
/* 116 */     this.iClassLoader.setSourceFinder(new PathResourceFinder(sourcePath));
/*     */   }
/*     */   
/*     */   public void setSourceFileCharacterEncoding(String optionalCharacterEncoding) {
/* 120 */     this.iClassLoader.setCharacterEncoding(optionalCharacterEncoding);
/*     */   }
/*     */   
/*     */   public void setDebuggingInfo(boolean debugSource, boolean debugLines, boolean debugVars) {
/* 124 */     this.debugSource = debugSource;
/* 125 */     this.debugLines = debugLines;
/* 126 */     this.debugVars = debugVars;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public void setCompileErrorHandler(UnitCompiler.ErrorHandler optionalCompileErrorHandler)
/*     */   {
/* 133 */     this.iClassLoader.setCompileErrorHandler(optionalCompileErrorHandler);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public void setWarningHandler(WarningHandler optionalWarningHandler)
/*     */   {
/* 141 */     this.iClassLoader.setWarningHandler(optionalWarningHandler);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected Class findClass(String name)
/*     */     throws ClassNotFoundException
/*     */   {
/* 152 */     byte[] bytecode = (byte[])this.precompiledClasses.remove(name);
/* 153 */     if (bytecode == null)
/*     */     {
/*     */ 
/*     */ 
/* 157 */       Map bytecodes = generateBytecodes(name);
/* 158 */       if (bytecodes == null) throw new ClassNotFoundException(name);
/* 159 */       this.precompiledClasses.putAll(bytecodes);
/*     */       
/*     */ 
/*     */ 
/* 163 */       bytecode = (byte[])this.precompiledClasses.remove(name);
/* 164 */       if (bytecode == null) {
/* 165 */         throw new JaninoRuntimeException("SNO: Scanning, parsing and compiling class \"" + name + "\" did not create a class file!?");
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 173 */     return defineBytecode(name, bytecode);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 181 */   Map precompiledClasses = new HashMap();
/*     */   
/*     */ 
/*     */ 
/*     */   private final JavaSourceIClassLoader iClassLoader;
/*     */   
/*     */ 
/*     */ 
/*     */   protected Map generateBytecodes(String name)
/*     */     throws ClassNotFoundException
/*     */   {
/* 192 */     if (this.iClassLoader.loadIClass(Descriptor.fromClassName(name)) == null) { return null;
/*     */     }
/* 194 */     Map bytecodes = new HashMap();
/* 195 */     Set compiledUnitCompilers = new HashSet();
/*     */     
/*     */ 
/* 198 */     Iterator it = this.unitCompilers.iterator(); for (;;) { if (!it.hasNext()) return bytecodes;
/* 199 */       UnitCompiler uc = (UnitCompiler)it.next();
/* 200 */       if (!compiledUnitCompilers.contains(uc)) {
/*     */         ClassFile[] cfs;
/*     */         try {
/* 203 */           cfs = uc.compileUnit(this.debugSource, this.debugLines, this.debugVars);
/*     */         } catch (CompileException ex) {
/* 205 */           throw new ClassNotFoundException("Compiling unit \"" + uc.compilationUnit.optionalFileName + "\"", ex);
/*     */         }
/*     */         
/*     */ 
/*     */ 
/* 210 */         for (int i = 0; i < cfs.length; i++) {
/* 211 */           ClassFile cf = cfs[i];
/* 212 */           bytecodes.put(cf.getThisClassName(), cf.toByteArray());
/*     */         }
/* 214 */         compiledUnitCompilers.add(uc);
/* 215 */         break;
/*     */       }
/*     */     }
/* 218 */     return bytecodes;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected Class defineBytecodes(String name, Map bytecodes)
/*     */   {
/* 231 */     Class clazz = null;
/* 232 */     for (Iterator it = bytecodes.entrySet().iterator(); it.hasNext();) {
/* 233 */       Map.Entry me = (Map.Entry)it.next();
/* 234 */       String name2 = (String)me.getKey();
/* 235 */       byte[] ba = (byte[])me.getValue();
/*     */       
/* 237 */       Class c = defineBytecode(name2, ba);
/* 238 */       if (name2.equals(name)) clazz = c;
/*     */     }
/* 240 */     return clazz;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected Class defineBytecode(String className, byte[] ba)
/*     */   {
/* 261 */     return defineClass(className, ba, 0, ba.length, this.optionalProtectionDomainFactory == null ? null : this.optionalProtectionDomainFactory.getProtectionDomain(ClassFile.getSourceResourceName(className)));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 270 */   protected boolean debugSource = Boolean.getBoolean("org.codehaus.janino.source_debugging.enable");
/* 271 */   protected boolean debugLines = this.debugSource;
/* 272 */   protected boolean debugVars = this.debugSource;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/* 277 */   private final Set unitCompilers = new HashSet();
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\janino\JavaSourceClassLoader.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       0.7.1
 */