package prefuse.action.assignment;

import java.util.Map;
import java.util.logging.Logger;
import prefuse.Visualization;
import prefuse.data.Table;
import prefuse.data.column.ColumnMetadata;
import prefuse.data.column.Columns;
import prefuse.data.tuple.TupleSet;
import prefuse.util.ColorLib;
import prefuse.util.ColorMap;
import prefuse.util.MathLib;
import prefuse.visual.VisualItem;

public class DataColorAction
  extends ColorAction
{
  private String m_dataField;
  private int m_type;
  private int m_scale = 0;
  private double[] m_dist;
  private int m_bins = -1;
  private Map m_omap;
  private ColorMap m_cmap = new ColorMap(null, 0.0D, 1.0D);
  private int[] m_palette;
  
  public DataColorAction(String paramString1, String paramString2, int paramInt, String paramString3)
  {
    super(paramString1, paramString3);
    setDataType(paramInt);
    setDataField(paramString2);
  }
  
  public DataColorAction(String paramString1, String paramString2, int paramInt, String paramString3, int[] paramArrayOfInt)
  {
    super(paramString1, paramString3);
    setDataType(paramInt);
    setDataField(paramString2);
    this.m_palette = paramArrayOfInt;
  }
  
  public String getDataField()
  {
    return this.m_dataField;
  }
  
  public void setDataField(String paramString)
  {
    this.m_dataField = paramString;
  }
  
  public int getDataType()
  {
    return this.m_type;
  }
  
  public void setDataType(int paramInt)
  {
    if ((paramInt < 0) || (paramInt >= 3)) {
      throw new IllegalArgumentException("Unrecognized data type: " + paramInt);
    }
    this.m_type = paramInt;
  }
  
  public int getScale()
  {
    return this.m_scale;
  }
  
  public void setScale(int paramInt)
  {
    if ((paramInt < 0) || (paramInt >= 4)) {
      throw new IllegalArgumentException("Unrecognized scale value: " + paramInt);
    }
    this.m_scale = paramInt;
  }
  
  public int getBinCount()
  {
    return this.m_bins;
  }
  
  public void setBinCount(int paramInt)
  {
    if ((this.m_scale == 3) && (paramInt <= 0)) {
      throw new IllegalArgumentException("The quantile scale can not be used without binning. Use a bin value greater than zero.");
    }
    this.m_bins = paramInt;
  }
  
  public void run(double paramDouble)
  {
    TupleSet localTupleSet = this.m_vis.getGroup(this.m_group);
    if (!(localTupleSet instanceof Table)) {
      return;
    }
    Table localTable = (Table)localTupleSet;
    setup(localTable);
    int i = this.m_scale;
    if ((this.m_scale == 3) && (this.m_bins <= 0))
    {
      Logger.getLogger(getClass().getName()).warning("Can't use quantilescale with no binning, defaulting to linear scale");
      this.m_scale = 0;
    }
    super.run(paramDouble);
    this.m_scale = i;
  }
  
  protected void setup(Table paramTable)
  {
    int i = 64;
    int[] arrayOfInt = this.m_palette;
    switch (this.m_type)
    {
    case 0: 
    case 1: 
      this.m_dist = getDistribution(paramTable);
      i = this.m_omap.size() - 1;
      arrayOfInt = this.m_palette != null ? this.m_palette : createPalette(i);
      this.m_cmap.setColorPalette(arrayOfInt);
      this.m_cmap.setMinValue(this.m_dist[0]);
      this.m_cmap.setMaxValue(this.m_dist[1]);
      return;
    case 2: 
      this.m_dist = getDistribution(paramTable);
      i = this.m_bins > 0 ? this.m_bins : i;
      arrayOfInt = this.m_palette != null ? this.m_palette : createPalette(i);
      this.m_cmap.setColorPalette(arrayOfInt);
      this.m_cmap.setMinValue(0.0D);
      this.m_cmap.setMaxValue(1.0D);
      return;
    }
  }
  
  protected double[] getDistribution(Table paramTable)
  {
    ColumnMetadata localColumnMetadata = paramTable.getMetadata(this.m_dataField);
    if (this.m_type == 2)
    {
      this.m_omap = null;
      if ((this.m_scale == 3) && (this.m_bins > 0))
      {
        arrayOfDouble = Columns.toDoubleArray(paramTable, this.m_dataField);
        return MathLib.quantiles(this.m_bins, arrayOfDouble);
      }
      double[] arrayOfDouble = new double[2];
      arrayOfDouble[0] = paramTable.getDouble(localColumnMetadata.getMinimumRow(), this.m_dataField);
      arrayOfDouble[1] = paramTable.getDouble(localColumnMetadata.getMaximumRow(), this.m_dataField);
      return arrayOfDouble;
    }
    this.m_omap = localColumnMetadata.getOrdinalMap();
    return new double[] { 0.0D, this.m_omap.size() - 1 };
  }
  
  protected int[] createPalette(int paramInt)
  {
    switch (this.m_type)
    {
    case 0: 
      return ColorLib.getCategoryPalette(paramInt);
    }
    return ColorLib.getGrayscalePalette(paramInt);
  }
  
  public int getColor(VisualItem paramVisualItem)
  {
    switch (this.m_type)
    {
    case 2: 
      double d1 = paramVisualItem.getDouble(this.m_dataField);
      double d2 = MathLib.interp(this.m_scale, d1, this.m_dist);
      return this.m_cmap.getColor(d2);
    }
    Integer localInteger = (Integer)this.m_omap.get(paramVisualItem.get(this.m_dataField));
    return this.m_cmap.getColor(localInteger.doubleValue());
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\action\assignment\DataColorAction.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */