package prefuse.action.layout.graph;

import java.awt.Dimension;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Rectangle2D.Double;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import prefuse.Display;
import prefuse.Visualization;
import prefuse.data.Schema;
import prefuse.data.Table;
import prefuse.data.util.TreeNodeIterator;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;

public class SquarifiedTreeMapLayout
  extends TreeLayout
{
  public static final String AREA = "_area";
  public static final Schema AREA_SCHEMA = new Schema();
  private static Comparator s_cmp = new Comparator()
  {
    public int compare(Object paramAnonymousObject1, Object paramAnonymousObject2)
    {
      double d1 = ((VisualItem)paramAnonymousObject1).getDouble("_area");
      double d2 = ((VisualItem)paramAnonymousObject2).getDouble("_area");
      return d1 < d2 ? -1 : d1 > d2 ? 1 : 0;
    }
  };
  private ArrayList m_kids = new ArrayList();
  private ArrayList m_row = new ArrayList();
  private Rectangle2D m_r = new Rectangle2D.Double();
  private double m_frame;
  
  public SquarifiedTreeMapLayout(String paramString)
  {
    this(paramString, 0.0D);
  }
  
  public SquarifiedTreeMapLayout(String paramString, double paramDouble)
  {
    super(paramString);
    this.m_frame = paramDouble;
  }
  
  public void run(double paramDouble)
  {
    NodeItem localNodeItem = getLayoutRoot();
    Rectangle2D localRectangle2D = getLayoutBounds();
    this.m_r.setRect(localRectangle2D.getX(), localRectangle2D.getY(), localRectangle2D.getWidth() - 1.0D, localRectangle2D.getHeight() - 1.0D);
    computeAreas(localNodeItem);
    setX(localNodeItem, null, 0.0D);
    setY(localNodeItem, null, 0.0D);
    localNodeItem.setBounds(0.0D, 0.0D, this.m_r.getWidth(), this.m_r.getHeight());
    updateArea(localNodeItem, this.m_r);
    layout(localNodeItem, this.m_r);
  }
  
  private void computeAreas(NodeItem paramNodeItem)
  {
    int i = 0;
    paramNodeItem.getTable().addColumns(AREA_SCHEMA);
    TreeNodeIterator localTreeNodeIterator = new TreeNodeIterator(paramNodeItem);
    while (localTreeNodeIterator.hasNext())
    {
      localObject = (NodeItem)localTreeNodeIterator.next();
      ((NodeItem)localObject).setDouble("_area", 0.0D);
    }
    localTreeNodeIterator = new TreeNodeIterator(paramNodeItem);
    while (localTreeNodeIterator.hasNext())
    {
      localObject = (NodeItem)localTreeNodeIterator.next();
      if (((NodeItem)localObject).getChildCount() == 0)
      {
        d1 = ((NodeItem)localObject).getSize();
        ((NodeItem)localObject).setDouble("_area", d1);
        for (NodeItem localNodeItem1 = (NodeItem)((NodeItem)localObject).getParent(); localNodeItem1 != null; localNodeItem1 = (NodeItem)localNodeItem1.getParent())
        {
          localNodeItem1.setDouble("_area", d1 + localNodeItem1.getDouble("_area"));
          localNodeItem1.setInteractive(false);
        }
        i++;
      }
    }
    Object localObject = this.m_vis.getDisplay(0).getSize();
    double d1 = (((Dimension)localObject).width - 1) * (((Dimension)localObject).height - 1);
    double d2 = d1 / i;
    localTreeNodeIterator = new TreeNodeIterator(paramNodeItem);
    while (localTreeNodeIterator.hasNext())
    {
      NodeItem localNodeItem2 = (NodeItem)localTreeNodeIterator.next();
      localNodeItem2.setDouble("_area", localNodeItem2.getDouble("_area") * d2);
    }
  }
  
  private void layout(NodeItem paramNodeItem, Rectangle2D paramRectangle2D)
  {
    Iterator localIterator = paramNodeItem.children();
    while (localIterator.hasNext()) {
      this.m_kids.add(localIterator.next());
    }
    Collections.sort(this.m_kids, s_cmp);
    double d = Math.min(paramRectangle2D.getWidth(), paramRectangle2D.getHeight());
    squarify(this.m_kids, this.m_row, d, paramRectangle2D);
    this.m_kids.clear();
    localIterator = paramNodeItem.children();
    while (localIterator.hasNext())
    {
      NodeItem localNodeItem = (NodeItem)localIterator.next();
      if (localNodeItem.getChildCount() > 0)
      {
        updateArea(localNodeItem, paramRectangle2D);
        layout(localNodeItem, paramRectangle2D);
      }
    }
  }
  
  private void updateArea(NodeItem paramNodeItem, Rectangle2D paramRectangle2D)
  {
    Rectangle2D localRectangle2D = paramNodeItem.getBounds();
    if (this.m_frame == 0.0D)
    {
      paramRectangle2D.setRect(localRectangle2D);
      return;
    }
    double d1 = 2.0D * this.m_frame * (localRectangle2D.getWidth() + localRectangle2D.getHeight() - 2.0D * this.m_frame);
    double d2 = paramNodeItem.getDouble("_area") - d1;
    double d3 = 0.0D;
    Iterator localIterator = paramNodeItem.children();
    while (localIterator.hasNext()) {
      d3 += ((NodeItem)localIterator.next()).getDouble("_area");
    }
    double d4 = d2 / d3;
    localIterator = paramNodeItem.children();
    while (localIterator.hasNext())
    {
      NodeItem localNodeItem = (NodeItem)localIterator.next();
      localNodeItem.setSize(localNodeItem.getDouble("_area") * d4);
    }
    paramRectangle2D.setRect(localRectangle2D.getX() + this.m_frame, localRectangle2D.getY() + this.m_frame, localRectangle2D.getWidth() - 2.0D * this.m_frame, localRectangle2D.getHeight() - 2.0D * this.m_frame);
  }
  
  private void squarify(List paramList1, List paramList2, double paramDouble, Rectangle2D paramRectangle2D)
  {
    double d1 = Double.MAX_VALUE;
    int i;
    while ((i = paramList1.size()) > 0)
    {
      paramList2.add(paramList1.get(i - 1));
      double d2 = worst(paramList2, paramDouble);
      if (d2 <= d1)
      {
        paramList1.remove(i - 1);
        d1 = d2;
      }
      else
      {
        paramList2.remove(paramList2.size() - 1);
        paramRectangle2D = layoutRow(paramList2, paramDouble, paramRectangle2D);
        paramDouble = Math.min(paramRectangle2D.getWidth(), paramRectangle2D.getHeight());
        paramList2.clear();
        d1 = Double.MAX_VALUE;
      }
    }
    if (paramList2.size() > 0)
    {
      paramRectangle2D = layoutRow(paramList2, paramDouble, paramRectangle2D);
      paramList2.clear();
    }
  }
  
  private double worst(List paramList, double paramDouble)
  {
    double d1 = Double.MIN_VALUE;
    double d2 = Double.MAX_VALUE;
    double d3 = 0.0D;
    Iterator localIterator = paramList.iterator();
    while (localIterator.hasNext())
    {
      double d4 = ((VisualItem)localIterator.next()).getDouble("_area");
      d2 = Math.min(d2, d4);
      d1 = Math.max(d1, d4);
      d3 += d4;
    }
    d3 *= d3;
    paramDouble *= paramDouble;
    return Math.max(paramDouble * d1 / d3, d3 / (paramDouble * d2));
  }
  
  private Rectangle2D layoutRow(List paramList, double paramDouble, Rectangle2D paramRectangle2D)
  {
    double d1 = 0.0D;
    Iterator localIterator = paramList.iterator();
    while (localIterator.hasNext()) {
      d1 += ((VisualItem)localIterator.next()).getDouble("_area");
    }
    double d2 = paramRectangle2D.getX();
    double d3 = paramRectangle2D.getY();
    double d4 = 0.0D;
    double d5 = paramDouble == 0.0D ? 0.0D : d1 / paramDouble;
    int i = paramDouble == paramRectangle2D.getWidth() ? 1 : 0;
    localIterator = paramList.iterator();
    while (localIterator.hasNext())
    {
      NodeItem localNodeItem1 = (NodeItem)localIterator.next();
      NodeItem localNodeItem2 = (NodeItem)localNodeItem1.getParent();
      if (i != 0)
      {
        setX(localNodeItem1, localNodeItem2, d2 + d4);
        setY(localNodeItem1, localNodeItem2, d3);
      }
      else
      {
        setX(localNodeItem1, localNodeItem2, d2);
        setY(localNodeItem1, localNodeItem2, d3 + d4);
      }
      double d6 = localNodeItem1.getDouble("_area") / d5;
      if (i != 0)
      {
        setNodeDimensions(localNodeItem1, d6, d5);
        d4 += d6;
      }
      else
      {
        setNodeDimensions(localNodeItem1, d5, d6);
        d4 += d6;
      }
    }
    if (i != 0) {
      paramRectangle2D.setRect(d2, d3 + d5, paramRectangle2D.getWidth(), paramRectangle2D.getHeight() - d5);
    } else {
      paramRectangle2D.setRect(d2 + d5, d3, paramRectangle2D.getWidth() - d5, paramRectangle2D.getHeight());
    }
    return paramRectangle2D;
  }
  
  private void setNodeDimensions(NodeItem paramNodeItem, double paramDouble1, double paramDouble2)
  {
    paramNodeItem.setBounds(paramNodeItem.getX(), paramNodeItem.getY(), paramDouble1, paramDouble2);
  }
  
  static
  {
    AREA_SCHEMA.addColumn("_area", Double.TYPE);
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\action\layout\graph\SquarifiedTreeMapLayout.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */